/*
Copyright (C) 2003 Bj\"orn Hoffmeister

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package de.uni_luebeck.tcs.demo.datamining;

import de.uni_luebeck.tcs.matlab.*;

/**
 Distribution Of Two Gaussians
 */
public class CompositNormalDist {
//constants -----------------------------------------------------------------------------------------------------------
	//private
	//protected
	// Parameter fuer die x-Achse
	protected final static double INTERVAL_BEGIN = -3.5;
	protected final static double INTERVAL_END = 3.5;
	
	//public
	public final static int RECOMMEND_RESOLUTION = 700; // => step = 0.01 per pdf

//inner classes -------------------------------------------------------------------------------------------------------
	
//attributes ----------------------------------------------------------------------------------------------------------
	//private
	//protected
	protected double dblMu0 = 0.0;
	protected double dblMu1 = 0.0;
	protected double dblSigma = 1.0;
		
	//public

//constructors --------------------------------------------------------------------------------------------------------
	//private
	//protected
	//public
	public CompositNormalDist() {}

//methods -------------------------------------------------------------------------------------------------------------
	//private
	//protected
	//public
	public double[] calcX(int n) {
		double[] dblX = new double[2 * n];
		double dblStep = (INTERVAL_END - INTERVAL_BEGIN) / (new Integer(n - 1)).doubleValue();
		int i = 0;
		for (double d = INTERVAL_BEGIN; d <= INTERVAL_END; d += dblStep) {
			dblX[i++] = dblMu0 + dblSigma * d;
			dblX[i++] = dblMu1 + dblSigma * d;
		}
		Sort.sort(dblX);
		return dblX;
	}	
	
	public double[] calcPDF(double[] dblX) {
		double[] dblPDF = new double[dblX.length];
		for (int i = 0; i < dblX.length; i++) {
			dblPDF[i] = 
				(0.5 / dblSigma) * 
				(NormalDist.normPDF((dblX[i] - dblMu0) / dblSigma) + 
				 NormalDist.normPDF((dblX[i] - dblMu1) / dblSigma));
		}
		return dblPDF;
	}

	public double[] calcRandomSample(int n) {
		double[] dblSample = new double[n];
		for (int i = 0; i < n; i++) {
			double dblY = Math.random();
			dblSample[i] = ((dblY <= 0.5)?
				dblMu0 + dblSigma * NormalDist.norminv(dblY / 0.5):
				dblMu1 + dblSigma * NormalDist.norminv((1.0 - dblY) / 0.5)
			);
		}
		Sort.sort(dblSample);		
		return dblSample;
	}

//interface -----------------------------------------------------------------------------------------------------------

//listener (public) ---------------------------------------------------------------------------------------------------

//getter-/setter-methods (public) -------------------------------------------------------------------------------------
	public void setMu0(double dblMu0) { this.dblMu0 = dblMu0; }	
	public double getMu0() { return this.dblMu0; }
	
	public void setMu1(double dblMu1) { this.dblMu1 = dblMu1; }	
	public double getMu1() { return this.dblMu1; }
	
	public void setSigma(double dblSigma) { this.dblSigma = dblSigma; }
	public double getSigma() { return this.dblSigma; }
}
