/*
Copyright (C) 2003 Bj\"orn Hoffmeister

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package de.uni_luebeck.tcs.demo.datamining;

import java.io.*;
import java.util.*;

/**
 EM-Algorithm for Estimating Means Of Two Gaussians
 */
public class EM4EM {
//constants -----------------------------------------------------------------------------------------------------------
	//private
	//protected
	protected static double ALPHA = 0.001;
	//public

//inner classes -------------------------------------------------------------------------------------------------------
	static class EMHistory {
		public double dblMu0 = 0.0;
		public double dblMu1 = 0.0;
		public double dblSigma = 0.0;
		
		public EMHistory(double dblMu0, double dblMu1, double dblSigma) {
			this.dblMu0 = dblMu0;
			this.dblMu1 = dblMu1;
			this.dblSigma = dblSigma;
		}
	}							
	
//attributes ----------------------------------------------------------------------------------------------------------
	//private
	//protected	
	//public

//constructors --------------------------------------------------------------------------------------------------------
	//private
	//protected
	//public

//methods -------------------------------------------------------------------------------------------------------------
	//private
	//protected
	//public	
	public static Vector calcEMHistory(double[] dblData, double dblSigma) {
		Vector vctHistory = new Vector();
		double dblMu0 = 0.0;
		double dblMu1 = 0.0;
		double dblPreviousMu0 = dblMu0;
		double dblPreviousMu1 = dblMu1;
		double dblPreviousSigma = dblSigma;
		for (int i = 0; i < dblData.length; i++) {
			if (dblData[i] < dblMu0) dblMu0 = dblData[i];
			if (dblData[i] > dblMu1) dblMu1 = dblData[i];
		}
		vctHistory.add(new EMHistory(dblMu0, dblMu1, Double.NaN));
		double dblSigmaFactor = - (1.0 / (2.0 * dblSigma * dblSigma));
		double dblSigmaQuotient = (new Integer(dblData.length - 1)).doubleValue();
		do {
			dblPreviousMu0 = dblMu0;
			dblPreviousMu1 = dblMu1;
			dblPreviousSigma = dblSigma;
			double dblSumE0 = 0.0;
			double dblSumE1 = 0.0;
			double dblSumEX0 = 0.0;
			double dblSumEX1 = 0.0;
			for (int i = 0; i < dblData.length; i++) {
				double dblQDiff0 = (dblData[i] - dblMu0) * (dblData[i] - dblMu0);
				double dblQDiff1 = (dblData[i] - dblMu1) * (dblData[i] - dblMu1);
				double dblGauss0 = Math.exp(dblSigmaFactor * dblQDiff0);
				double dblGauss1 = Math.exp(dblSigmaFactor * dblQDiff1);
				double dblE0 = dblGauss0 / (dblGauss0 + dblGauss1);
				double dblE1 = dblGauss1 / (dblGauss0 + dblGauss1);
				dblSumE0 += dblE0;
				dblSumE1 += dblE1;
				dblSumEX0 += dblE0 * dblData[i];
				dblSumEX1 += dblE1 * dblData[i];
				dblSigma += dblE0 * dblQDiff0 + dblE1 * dblQDiff1;
			}
			dblMu0 = dblSumEX0 / dblSumE0;
			dblMu1 = dblSumEX1 / dblSumE1;
			dblSigma /= dblSigmaQuotient;
			vctHistory.add(new EMHistory(dblMu0, dblMu1, dblSigma));			
		} while (Math.abs(dblPreviousSigma - dblSigma) > ALPHA);
//		} while ((Math.abs(dblPreviousMu0 - dblMu0) > ALPHA) || (Math.abs(dblPreviousMu1 - dblMu1) > ALPHA));
		return vctHistory;				
	}	
	
	public static double[] readFile(String strFile) throws IOException, NumberFormatException {
		BufferedReader bfrFile = new BufferedReader(new FileReader(strFile));
		Vector vctData = new Vector();
		String strTmp = null;
		do {
			strTmp = bfrFile.readLine();
			if (strTmp != null) try {
				vctData.add(new Double(strTmp));
			} catch (NumberFormatException ignore) {}			
		} while (strTmp != null);
		
		double[] dblData = new double[vctData.size()];
		for (int i = 0; i < vctData.size(); i++) {
			dblData[i] = ((Double)vctData.get(i)).doubleValue();
		}
		return dblData;		
	}	

	public static void main(String[] args) throws Exception {
		double[] dblData = null;
		double dblSigma = 1.0;
		if (args.length == 0) {
			System.out.println("java EM4EM <arff-file> [<sigma>]");
			System.exit(0);
		}		
		dblData = readFile(args[0]);		
		if (args.length > 1) {
			dblSigma = (new Double(args[1])).doubleValue();
		}
		
		Vector vctEMHistory = calcEMHistory(dblData, dblSigma);
		for (int i = 0; i < vctEMHistory.size(); i++) {
			EMHistory emhTmp = (EMHistory)vctEMHistory.get(i);
			System.out.println(i + ": mu0=" + emhTmp.dblMu0 + "; mu1=" + emhTmp.dblMu1 + "; sigma=" + emhTmp.dblSigma);
		}
	}

//interface -----------------------------------------------------------------------------------------------------------

//listener (public) ---------------------------------------------------------------------------------------------------

//getter-/setter-methods (public) -------------------------------------------------------------------------------------
	
}
