/*
EMApplet
Copyright (C) 2003 Bj\"orn Hoffmeister

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package de.uni_luebeck.tcs.demo.datamining;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

// import de.uni_luebeck.tcs.matlab.*;
import de.uni_luebeck.tcs.visualization.*;

public class EMApplet extends JApplet {
//constants -----------------------------------------------------------------------------------------------------------
	//private
	private static final double N_MIN = 0.0;
	private static final double N_AVG = 500.0;
	private static final double N_MAX = 1000.0;
	private static final double N_DEFAULT = 100.0;
	private static final double N_MIN_VALUE = 10.0;
	private static final double N_MAX_VALUE = Double.POSITIVE_INFINITY;
	private static final double N_STEP = 1;

	private static final double MU0_MIN = -3.0;
	private static final double MU0_AVG = 0.0;
	private static final double MU0_MAX = 3.0;
	private static final double MU0_DEFAULT = -1.0;
	private static final double MU0_MIN_VALUE = Double.NEGATIVE_INFINITY;
	private static final double MU0_MAX_VALUE = Double.POSITIVE_INFINITY;
	private static final double MU0_STEP = 0.1;
	
	private static final double MU1_MIN = -3.0;
	private static final double MU1_AVG = 0.0;
	private static final double MU1_MAX = 3.0;
	private static final double MU1_DEFAULT = 1.0;
	private static final double MU1_MIN_VALUE = Double.NEGATIVE_INFINITY;
	private static final double MU1_MAX_VALUE = Double.POSITIVE_INFINITY;
	private static final double MU1_STEP = 0.1;
	
	private static final double SIGMA_MIN = 0.0;
	private static final double SIGMA_AVG = 1.0;
	private static final double SIGMA_MAX = 2.0;
	private static final double SIGMA_DEFAULT = 1.0;
	private static final double SIGMA_MIN_VALUE = 0.0;
	private static final double SIGMA_MAX_VALUE = Double.POSITIVE_INFINITY;
	private static final double SIGMA_STEP = 0.1;
	
	//protected
	//public

//inner classes -------------------------------------------------------------------------------------------------------
	
//attributes ----------------------------------------------------------------------------------------------------------
	//private
	//protected
	// PDFTab
	protected JPanel jplPDF = null;
	protected JRangedInput.Model mdlMu0 = null;
	protected JRangedInput.Model mdlMu1 = null;
	protected JRangedInput.Model mdlSigma = null;
	protected JButton jbtPDF = null;
	protected Axes axsPDF = null;	

	// EMTab
	protected JPanel jplEM = null;
	protected JRangedInput.Model mdlN = null;
	protected JButton jbtSim = null;
	protected JRangedInput.Model mdlIteration = null;
	protected JRangedInput.Constraints jrcIteration = null;
	protected JButton jbtFirst = null;
	protected JButton jbtLast = null;
	protected JButton jbtPrevious = null;
	protected JButton jbtNext = null;
	protected Axes axsEM = null;
	protected JTable jtbTable = null;

	// EM
	protected Vector vctEMHistory = null;
	protected int intIteration = 0;
	protected Double dblEMMu0AsObject = null;
	protected Double dblEMMu1AsObject = null;
	protected Double dblEMSigmaAsObject = null;

	protected double[] dblX = null;
	protected double[] dblPDF = null;
	protected double[] dblEM = null;
	protected double[] dblSample = null;
	
	protected CompositNormalDist cndPDF = null;
	protected CompositNormalDist cndEM = null;

	//public

//constructors --------------------------------------------------------------------------------------------------------
	//private
	//protected
	//public

//methods -------------------------------------------------------------------------------------------------------------
	//private
	//protected
	protected void firstIteration() {
		mdlIteration.setX(jrcIteration.lowerBound);
		updateIteration();
	}	

	protected void lastIteration() {
		mdlIteration.setX(jrcIteration.upperBound);
		updateIteration();
	}

	protected void previousIteration() {
		mdlIteration.setX(mdlIteration.getX() - 1.0);
		updateIteration();
	}

	protected void nextIteration() {
		mdlIteration.setX(mdlIteration.getX() + 1.0);
		updateIteration();
	}
	protected void updateIteration() {
		double dblIteration = mdlIteration.getX();
		intIteration = (new Double(dblIteration)).intValue();
	
		if (!jbtFirst.isEnabled()) if (dblIteration > jrcIteration.lowerBound) {
			jbtFirst.setEnabled(true);
			jbtPrevious.setEnabled(true);
		}
		if (!jbtLast.isEnabled()) if (dblIteration < jrcIteration.upperBound) {
			jbtLast.setEnabled(true);
			jbtNext.setEnabled(true);
		}
		if (jbtFirst.isEnabled()) if (dblIteration == jrcIteration.lowerBound) {
			jbtFirst.setEnabled(false);
			jbtPrevious.setEnabled(false);
		}
		if (jbtLast.isEnabled()) if (dblIteration == jrcIteration.upperBound) {
			jbtLast.setEnabled(false);
			jbtNext.setEnabled(false);
		}
		
		updateEM();
	}		

	protected void changedPDFParameter() {
		if (!jbtPDF.isEnabled()) jbtPDF.setEnabled(true);
	}			

	protected void updatePDF() {
		jbtPDF.setEnabled(false);
		cndPDF.setMu0(mdlMu0.getX());
		cndPDF.setMu1(mdlMu1.getX());
		cndPDF.setSigma(mdlSigma.getX());
		cndEM.setSigma(mdlSigma.getX());
		
		this.dblX = cndPDF.calcX(CompositNormalDist.RECOMMEND_RESOLUTION);
		this.dblPDF = cndPDF.calcPDF(this.dblX);
		
		updatePDFAxes();
		updateSample();
	}

	protected void updateSample() {
		// simulate sample
		int intN = (new Double(Math.floor(mdlN.getX()))).intValue();
		this.dblSample = cndPDF.calcRandomSample(intN);
		// calculate EMHistory
		this.vctEMHistory = EM4EM.calcEMHistory(dblSample, mdlSigma.getX());
		// set History-Controls
		this.intIteration = vctEMHistory.size() - 1;
		double dblMax = (new Integer(intIteration)).doubleValue();
		try {
			jrcIteration = new JRangedInput.Constraints(
				0.0,
				dblMax,
				dblMax / 2,
				dblMax,
				0.0,
				dblMax,
				1.0,
				JRangedInput.INTEGER_MODE
			);
		} catch(Exception e) {}		
		mdlIteration.setConstraints(jrcIteration);
		mdlIteration.setX(jrcIteration.upperBound);
		if (!jbtFirst.isEnabled()) {
			jbtFirst.setEnabled(true);
			jbtPrevious.setEnabled(true);
		}
		if (jbtLast.isEnabled()) {
			jbtLast.setEnabled(false);
			jbtNext.setEnabled(false);
		}
		// update
		updateEM();		
	}

	protected void updateEM() {
		EM4EM.EMHistory emhTmp = (EM4EM.EMHistory)vctEMHistory.get(intIteration);
		
		this.dblEMMu0AsObject = new Double(emhTmp.dblMu0);
		this.dblEMMu1AsObject = new Double(emhTmp.dblMu1);
		this.dblEMSigmaAsObject = new Double(emhTmp.dblSigma);
		
		this.cndEM.setMu0(emhTmp.dblMu0);
		this.cndEM.setMu1(emhTmp.dblMu1);
		
		this.dblEM = cndEM.calcPDF(this.dblX);
		
		updateEMAxes();		
	}

	public void updatePDFAxes() {
		try {
			axsPDF.removeAllLines();

			FilledGraph fgpPDF = new FilledGraph(dblX, dblPDF);
			fgpPDF.setColor(Color.green);
			fgpPDF.setFillColor(new Color(
					Color.green.getRed(), 
					Color.green.getGreen(), 
					Color.green.getBlue(), 
					Color.green.getAlpha() / 8
				));
			axsPDF.addLine(fgpPDF);		
        	
        		// Abmessungen der Achsen setzen
        		double[] dblXScale = new double[] {dblX[0], dblX[dblX.length - 1]};
        		String[] strXScale = new String[] {"", ""}; 
 
        		axsPDF.setXAuto(Axes.AUTO);
       			axsPDF.setXIndex(dblXScale);
        		axsPDF.setXIndexLabel(strXScale);
        		axsPDF.setXAuto(Axes.MANUAL);
        	
       			double dblMaxPDF = 1.1 * fgpPDF.getMaxY();
        		double[] dblPDFScale = createScale(0.0, dblMaxPDF);
       			String[] strPDFScale = createLabel(dblPDFScale);
       	
        		axsPDF.setYAuto(Axes.AUTO);
       			axsPDF.setYIndex(dblPDFScale);
        		axsPDF.setYIndexLabel(strPDFScale);
        		axsPDF.setYAuto(Axes.MANUAL);
        	} catch(Exception ignore) {}
        	repaint();
        }
        
	protected void updateEMAxes() {
		try {
			axsEM.removeAllLines();
			Graph grpPDF = new Graph(dblX, dblPDF);
			grpPDF.setColor(new Color(
					Color.green.getRed(), 
					Color.green.getGreen(), 
					Color.green.getBlue(), 
					Color.green.getAlpha() / 2
				));
			FilledGraph fgpEM = new FilledGraph(dblX, dblEM);
			fgpEM.setColor(new Color(
					Color.blue.getRed(), 
					Color.blue.getGreen(), 
					Color.blue.getBlue(), 
					Color.blue.getAlpha() / 2
				));
			fgpEM.setFillColor(new Color(
					Color.blue.getRed(), 
					Color.blue.getGreen(), 
					Color.blue.getBlue(), 
					Color.blue.getAlpha() / 16
				));
	       		double dblMinEM = Math.min(grpPDF.getMinY(), fgpEM.getMinY());
	       		double dblMaxEM = 1.1 * Math.max(grpPDF.getMaxY(), fgpEM.getMaxY());

			Sample smpX = new Sample(dblSample, dblMinEM, dblMaxEM, 10);
			smpX.setColor(Color.black);
			
			Graph grpMu0 = new Graph(
					new double[]{mdlMu0.getX(), mdlMu0.getX()}, 
					new double[]{dblMinEM, dblMaxEM}
				);
			grpMu0.setColor(Color.green);
			Graph grpMu1 = new Graph(
					new double[]{mdlMu1.getX(), mdlMu1.getX()}, 
					new double[]{dblMinEM, dblMaxEM}
				);
			grpMu1.setColor(Color.green);
			Graph grpEMMu0 = new Graph(
					new double[]{dblEMMu0AsObject.doubleValue(), dblEMMu0AsObject.doubleValue()}, 
					new double[]{dblMinEM, dblMaxEM}
				);
			grpEMMu0.setColor(Color.blue);
			Graph grpEMMu1 = new Graph(
					new double[]{dblEMMu1AsObject.doubleValue(), dblEMMu1AsObject.doubleValue()}, 
					new double[]{dblMinEM, dblMaxEM}
				);
			grpEMMu1.setColor(Color.blue);
			
			axsEM.addLine(fgpEM);		
			axsEM.addLine(grpPDF);		
			axsEM.addLine(smpX);		
			axsEM.addLine(grpMu0);		
			axsEM.addLine(grpMu1);		
			axsEM.addLine(grpEMMu0);		
			axsEM.addLine(grpEMMu1);		

        		// Abmessungen der Achsen setzen
        		double[] dblXScale = new double[] {dblX[0], dblX[dblX.length - 1]};
        		String[] strXScale = new String[] {"", ""}; 
 
        		axsEM.setXAuto(Axes.AUTO);
       			axsEM.setXIndex(dblXScale);
        		axsEM.setXIndexLabel(strXScale);
        		axsEM.setXAuto(Axes.MANUAL);
        	
        		double[] dblEMScale = createScale(smpX.getMinY(), dblMaxEM);
       			String[] strEMScale = createLabel(dblEMScale);
       	
        		axsEM.setYAuto(Axes.AUTO);
       			axsEM.setYIndex(dblEMScale);
        		axsEM.setYIndexLabel(strEMScale);
        		axsEM.setYAuto(Axes.MANUAL);
        	} catch(Exception ignore) {}
        	repaint();
        }

	//public
	public void init() {
		JRangedInput.Constraints N_CONSTRAINTS = null;
		try {
			N_CONSTRAINTS = new JRangedInput.Constraints(
				N_MIN,
				N_MAX, 
				N_AVG, 
				N_DEFAULT, 
				N_MIN_VALUE, 
				N_MAX_VALUE, 
				N_STEP,
				JRangedInput.INTEGER_MODE
			);
		} catch(Exception e) {}
		JRangedInput.Constraints MU0_CONSTRAINTS = null;
		try {
			MU0_CONSTRAINTS = new JRangedInput.Constraints(
				MU0_MIN,
				MU0_MAX, 
				MU0_AVG, 
				MU0_DEFAULT, 
				MU0_MIN_VALUE, 
				MU0_MAX_VALUE, 
				MU0_STEP,
				JRangedInput.DOUBLE_MODE
			);
		} catch(Exception e) {}
		JRangedInput.Constraints MU1_CONSTRAINTS = null;
		try {
			MU1_CONSTRAINTS = new JRangedInput.Constraints(
				MU1_MIN,
				MU1_MAX, 
				MU1_AVG, 
				MU1_DEFAULT, 
				MU1_MIN_VALUE, 
				MU1_MAX_VALUE, 
				MU1_STEP,
				JRangedInput.DOUBLE_MODE
			);
		} catch(Exception e) {}
		JRangedInput.Constraints SIGMA_CONSTRAINTS = null;
		try {
			SIGMA_CONSTRAINTS = new JRangedInput.Constraints(
				SIGMA_MIN,
				SIGMA_MAX, 
				SIGMA_AVG, 
				SIGMA_DEFAULT, 
				SIGMA_MIN_VALUE, 
				SIGMA_MAX_VALUE, 
				SIGMA_STEP,
				JRangedInput.DOUBLE_MODE
			);
		} catch(Exception e) {}
		jrcIteration = null;
		try {
			jrcIteration = new JRangedInput.Constraints(
				0.0,
				10.0, 
				5.0,
				10.0,
				0.0,
				10.0, 
				1.0,
				JRangedInput.INTEGER_MODE
			);
		} catch(Exception e) {}

		JPanel jplContent = new JPanel();
		jplContent.setLayout(new BorderLayout());		
		JTabbedPane jtpTab = new JTabbedPane();

		GridBagConstraints gbcJRangedInput = new GridBagConstraints();
			gbcJRangedInput.anchor = GridBagConstraints.CENTER;
			gbcJRangedInput.fill = GridBagConstraints.HORIZONTAL;
			gbcJRangedInput.insets = new Insets(5, 0, 0, 0);
			gbcJRangedInput.gridwidth = GridBagConstraints.REMAINDER;
			gbcJRangedInput.weightx = 1.0;			
		GridBagConstraints gbcButton = new GridBagConstraints();
			gbcButton.anchor = GridBagConstraints.SOUTH;
			gbcButton.fill = GridBagConstraints.HORIZONTAL;
			gbcButton.insets = new Insets(5, 5, 5, 5);
			gbcButton.gridwidth = GridBagConstraints.REMAINDER;
			gbcButton.weightx = 2.0;
		GridBagConstraints gbcStretch = new GridBagConstraints();
			gbcStretch.fill = GridBagConstraints.BOTH;
			gbcStretch.gridwidth = GridBagConstraints.REMAINDER;
			gbcStretch.weighty = 2.0;
			gbcStretch.insets = new Insets(5, 5, 5, 5);			
		GridBagConstraints gbcParamPanel = new GridBagConstraints();
			gbcParamPanel.gridwidth = GridBagConstraints.REMAINDER;
			gbcParamPanel.fill = GridBagConstraints.HORIZONTAL;
			gbcParamPanel.insets = new Insets(5, 5, 5, 5);
		GridBagConstraints gbcTable = new GridBagConstraints();
			gbcTable.anchor = GridBagConstraints.NORTH;
			gbcTable.fill = GridBagConstraints.HORIZONTAL;
			gbcTable.gridwidth = GridBagConstraints.REMAINDER;
			gbcTable.weightx = 2.0;
			gbcTable.insets = new Insets(5, 63, 5, 30);

// Verteilungspanel
		jplPDF = new JPanel();
		jplPDF.setLayout(new BorderLayout());
		
		JPanel jplPDFAxes = new JPanel();
		jplPDFAxes.setLayout(new GridLayout(2, 1, 5, 5));

		JPanel jplPDFSwing = new JPanel();
		GridBagLayout gblPDFSwing = new GridBagLayout();
	        jplPDFSwing.setLayout(gblPDFSwing);
		
	// left side
		// jplPDFAxes
		axsPDF = new Axes(400, 200);
		try {
			axsPDF.setTitle("PDF");
			axsPDF.setXIndex(new double[]{-3.5, 3.5});
			axsPDF.setXIndexLabel(new String[]{"", ""});
		} catch(Exception ignore) {}
		jplPDFAxes.add(axsPDF);
		
	// right side
		// jplPDFSwing
		jbtPDF = new JButton("update");
			jbtPDF.setForeground((new JLabel()).getForeground());
			jbtPDF.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					updatePDF();
				}
			});
			jbtPDF.setEnabled(false);
					
		PropertyChangeListener pclPDF = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				changedPDFParameter();
			}
		};
		
		JRangedInput jriMu0 = new JRangedInput("mu0", MU0_CONSTRAINTS);
			mdlMu0 = jriMu0.getModel();
			mdlMu0.addPropertyChangeListener(pclPDF);
		JRangedInput jriMu1 = new JRangedInput("mu1", MU1_CONSTRAINTS);
			mdlMu1 = jriMu1.getModel();
			mdlMu1.addPropertyChangeListener(pclPDF);		
		JRangedInput jriSigma = new JRangedInput("sigma", SIGMA_CONSTRAINTS);
			mdlSigma = jriSigma.getModel();
			mdlSigma.addPropertyChangeListener(pclPDF);		
		
		JPanel jplPDFParam = new JPanel();
			jplPDFParam.setBorder(BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder(" PDF-Parameter: "),
				BorderFactory.createEmptyBorder(5,5,5,5))
			);
			GridBagLayout gblPDFParam = new GridBagLayout();		
			jplPDFParam.setLayout(gblPDFParam);
			gblPDFParam.setConstraints(jriMu0, gbcJRangedInput);
			gblPDFParam.setConstraints(jriMu1, gbcJRangedInput);
			gblPDFParam.setConstraints(jriSigma, gbcJRangedInput);
			jplPDFParam.add(jriMu0);
			jplPDFParam.add(jriMu1);
			jplPDFParam.add(jriSigma);			
		
		Component cmpPDFStretch = Box.createVerticalGlue();
		
		gblPDFSwing.setConstraints(jplPDFParam, gbcParamPanel);
		gblPDFSwing.setConstraints(jbtPDF, gbcButton);
		gblPDFSwing.setConstraints(cmpPDFStretch, gbcStretch);		
		jplPDFSwing.add(jplPDFParam);
		jplPDFSwing.add(jbtPDF);
		jplPDFSwing.add(cmpPDFStretch);
		
		// jplPDF
		jplPDF.add(jplPDFAxes, BorderLayout.CENTER);
		jplPDF.add(jplPDFSwing, BorderLayout.EAST);
		
// EM-Panel
		jplEM = new JPanel();
		jplEM.setLayout(new BorderLayout());
		
		JPanel jplEMAxes = new JPanel();
		jplEMAxes.setLayout(new GridLayout(2, 1, 5, 5));

		JPanel jplEMSwing = new JPanel();
		GridBagLayout gblEMSwing = new GridBagLayout();
	        jplEMSwing.setLayout(gblEMSwing);
		
	// left side
		// jplEMAxes
		axsEM = new Axes(400, 200);
		try {
			axsEM.setTitle("EM");
			axsEM.setXIndex(new double[]{-3.5, 3.5});
			axsEM.setXIndexLabel(new String[]{"", ""});
		} catch(Exception ignore) {}
		jplEMAxes.add(axsEM);
		
		JPanel jplBottom = new JPanel();
			GridBagLayout gblBottom = new GridBagLayout();
			jplBottom.setLayout(gblBottom);

		AbstractTableModel atmTable = new AbstractTableModel() {
			private String[] COLUMN_NAME = {null, "pdf", "EM"};
			private String[] ROW_NAME = {"mu0", "mu1", "sigma/std^2"};
			private Class[] COLUMN_CLASS = {String.class, Double.class, Double.class};
			
			public int getRowCount() { return 3; }
			
			public int getColumnCount() { return 3; }
			
			public Object getValueAt(int intRow, int intColumn) {
				switch (intColumn) {
				case 0:
					return ROW_NAME[intRow];
				case 1:
					switch (intRow) {
					case 0: return EMApplet.this.mdlMu0.getXAsObject();
					case 1: return EMApplet.this.mdlMu1.getXAsObject();
					case 2: return EMApplet.this.mdlSigma.getXAsObject();
					}
					break;
				case 2:
					switch (intRow) {
					case 0: return EMApplet.this.dblEMMu0AsObject;
					case 1: return EMApplet.this.dblEMMu1AsObject;
					case 2: return EMApplet.this.dblEMSigmaAsObject;
					}
					break;
				}
				return null;
			}
			
			public String getColumnName(int intColumn) {
				return COLUMN_NAME[intColumn];
			}

			public Class getColumnClass(int intColumn) {
				return COLUMN_CLASS[intColumn];
			}			
		};
		jtbTable = new JTable(atmTable);
		JPanel jplTable = new JPanel(new BorderLayout());		
			jplTable.setBorder(BorderFactory.createLineBorder(Color.black));
		jplTable.add(jtbTable.getTableHeader(), BorderLayout.NORTH);
		jplTable.add(jtbTable, BorderLayout.CENTER);			

		Component cmpTableStretch = Box.createVerticalGlue();
		
		gblBottom.setConstraints(jplTable, gbcTable);
		gblBottom.setConstraints(cmpTableStretch, gbcStretch);		
		jplBottom.add(jplTable);
		jplBottom.add(cmpTableStretch);

		jplEMAxes.add(jplBottom);
		
	// right side
		// jplEMSwing
		jbtSim = new JButton("simulate");
			jbtSim.setForeground((new JLabel()).getForeground());
			jbtSim.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					updateSample();
				}
			});

		JRangedInput jriN = new JRangedInput("n", N_CONSTRAINTS);
			mdlN = jriN.getModel();			
		
		JPanel jplN = new JPanel();
			jplN.setBorder(BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder(" sample: "),
				BorderFactory.createEmptyBorder(5,5,5,5))
			);
			jplN.add(jriN);		
		
		JRangedInput jriIteration = new JRangedInput("iteration", jrcIteration);
			mdlIteration = jriIteration.getModel();			
			mdlIteration.addPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent e) {
					updateIteration();
				}
			});
		
		jbtFirst = new JButton("|<");
			jbtFirst.setForeground((new JLabel()).getForeground());
			jbtFirst.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					firstIteration();
				}
			});
		jbtLast = new JButton(">|");
			jbtLast.setForeground((new JLabel()).getForeground());
			jbtLast.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					lastIteration();
				}
			});
		jbtPrevious = new JButton(" <");
			jbtPrevious.setForeground((new JLabel()).getForeground());
			jbtPrevious.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					previousIteration();
				}
			});			
		jbtNext = new JButton("> ");
			jbtNext.setForeground((new JLabel()).getForeground());
			jbtNext.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					nextIteration();
				}
			});
		JPanel jplControl = new JPanel();
		jplControl.setLayout(new GridLayout(1, 4, 1, 0));
		jplControl.add(jbtFirst);
		jplControl.add(jbtPrevious);
		jplControl.add(jbtNext);
		jplControl.add(jbtLast);
		
		JPanel jplIteration = new JPanel();
			jplIteration.setBorder(BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder(" iteration: "),
				BorderFactory.createEmptyBorder(5,5,5,5))
			);
			GridBagLayout gblIteration = new GridBagLayout();		
			jplIteration.setLayout(gblIteration);
			gblIteration.setConstraints(jriIteration, gbcJRangedInput);
			gblIteration.setConstraints(jplControl, gbcJRangedInput);
			jplIteration.add(jriIteration);
			jplIteration.add(jplControl);		
		
		Component cmpEMStretch = Box.createVerticalGlue();
		
		gblEMSwing.setConstraints(jplN, gbcParamPanel);
		gblEMSwing.setConstraints(jbtSim, gbcButton);
		gblEMSwing.setConstraints(cmpEMStretch, gbcStretch);		
		gblEMSwing.setConstraints(jplIteration, gbcParamPanel);
		jplEMSwing.add(jplN);
		jplEMSwing.add(jbtSim);
		jplEMSwing.add(cmpEMStretch);
		jplEMSwing.add(jplIteration);
		
		// jplSample
		jplEM.add(jplEMAxes, BorderLayout.CENTER);
		jplEM.add(jplEMSwing, BorderLayout.EAST);
			
// TabPanel zusammensetzen
		jtpTab.addTab("Verteilungsfunktion", jplPDF);
		jtpTab.addTab("EM-Algorithmus", jplEM);
		jtpTab.setSelectedIndex(1);
// jplContent
		jplContent.add(jtpTab, BorderLayout.CENTER);

// JApplet
		setContentPane(jplContent);
		
		
		cndPDF = new CompositNormalDist();
		cndEM = new CompositNormalDist();
		updatePDF();
	}

//	public void start() {
//		updatePDF();
//	}

//	public void stop() {
//		System.gc();
//	}

// Hilfsfunktionen
	// Erstellt eine Skala von min(dblStart, 0) bis dblEnd
	protected double[] createScale(double dblStart, double dblEnd) {
		final double[] dblDefault = new double[]{ 0.0, 0.25, 0.5, 0.75, 1.0};
		
		int n = (dblStart < dblDefault[0])?1:0;
		for (int i = 0; i < dblDefault.length; i++) if (dblDefault[i] < dblEnd) n++;
		double[] dblScale = new double[++n];
		int m = 0;
		if (dblStart < dblDefault[0]) dblScale[m++] = dblStart;
		int l = n - m - 1;
		for (int i = 0; i < l; i++) {
			dblScale[m++] = dblDefault[i];
		}
		dblScale[m] = dblEnd;
		return dblScale;
	}
	
	// Erstellt eine Beschriftung, in der neue die Zahlen auftauchen, die in
	// dblScale und dblDefault enthalten sind
	protected String[] createLabel(double[] dblScale) {
		final double[] dblDefault = new double[]{ 0.0, 0.25, 0.5, 0.75, 1.0};
		
		String[] strLabel = new String[dblScale.length];
		for (int i = 0; i < strLabel.length; i++) {
			boolean blnIn = false;
			for (int j = 0; j < dblDefault.length; j++) 
				if (dblScale[i] == dblDefault[j]) {
					blnIn = true;
					break;
				}
			if (blnIn) { strLabel[i] = Range2Range.double2String(dblScale[i], -1, 2); }
			else { strLabel[i] = ""; }
		}
		return strLabel;
	}

// interface ----------------------------------------------------------------------------------------------------------

//listener (public) ---------------------------------------------------------------------------------------------------

//getter-/setter-methods (public) -------------------------------------------------------------------------------------

// main ---------------------------------------------------------------------------------------------------------------

    public static void main(String[] args) {
	JApplet japEM = new EMApplet();
	JFrame jfrEM = new JFrame("EMDemo");
	jfrEM.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	//	jfrEM.setResizable(false);
	jfrEM.setContentPane(japEM);
	japEM.init();
	jfrEM.pack();
	jfrEM.setVisible(true);
	japEM.start();		
    }
}

