/*
Copyright (C) 2003 Bj\"orn Hoffmeister

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package de.uni_luebeck.tcs.matlab;

/**
* Data Analysis
* Basic Operations
*/
public class Basic {
// Maximum und Minimum
	public static double max(double[] d) {
		double max = d[0];
		for (int i = 1; i < d.length; i++) if (d[i] > max) max = d[i];
		return max;
	}
	
	public static double min(double[] d) {
		double min = d[0];
		for (int i = 1; i < d.length; i++) if (d[i] < min) min = d[i];
		return min;
	}
// Test auf NaN und Inf
	public static boolean isNaN(double[] d) {
		boolean b = false;
		for (int i = 1; i < d.length; i++) if (Double.isNaN(d[i])) b = true;
		return b;
	}
	
// Summen und Produkte
	/**
	* Cumulative sum of elements
	**/	
	public static void cumsum(double[] d) {
		for (int i = 1; i < d.length; i++) d[i] += d[i - 1];
	}
	
// Soritieralgorithmen 
	/**
	/* Quicksort fuer double-Arrays
	**/
	public static void sort(double[] d) {
		// Spezialfaelle behandeln
		if (d == null) return;
		if ((d.length == 0) || (d.length == 1)) return;
		sortrec(d, 0, d.length-1);
	}
	
	private static void sortrec(double[] d, int from, int to) {
		if (d.length == 1) return;
		int i = from;
		int j = to;
		double center = d[(from + to) / 2];
		do {
			while((i < to) && (center > d[i])) i++;
			while((j > from) && (center < d[j])) j--;
			if (i < j) {
				double temp = d[i];
				d[i] = d[j];
				d[j] = temp;
			}
			if (i <= j) {
				i++;
				j--;
			}
		} while(i <= j);
		if (from < j) sortrec(d, from, j);
		if (i < to) sortrec(d, i, to);
	}

// Statistische Methoden
	/**
	* Mittelwert
	**/
	public static double mean(double[] d) {
		if (d == null) return Double.NaN;
		if (d.length == 0) return Double.NaN;
		double sum = 0.0;
		for (int i = 0; i < d.length; i++) sum += d[i];
		return (sum / d.length);
	}
}

				
			
