/*
Copyright (C) 2003 Bj\"orn Hoffmeister

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package de.uni_luebeck.tcs.visualization;

import java.awt.Color;
import java.util.Vector;

/*
* Klasse die Line implementiert und ein paar Funktionen zur Verfuegung
* stellt, die haeufig von Line-Implementationen benoetigt werden
*/
public abstract class AbstractLine implements Line {
	private Vector vctChild = new Vector(3);
	private Color clrLine = null;
	protected double dblXMin = 0.0;
	protected double dblXMax = 0.0;
	protected double dblYMin = 0.0;
	protected double dblYMax = 0.0;
			
	public double getMinX() { return this.dblXMin; }
	
	public double getMaxX() { return this.dblXMax; }
	
	public double getMinY() { return this.dblYMin; }
	
	public double getMaxY() { return this.dblYMax; }
	
	// hier soll das zeichnen der linie implementiert sein
	public abstract void drawLine(AxesGraphicsInterface agr);
		
	public void draw(AxesGraphicsInterface agr) {
		Color clrTemp = null;
		// ggbf. Farbe setzen
		if (clrLine != null) {
			clrTemp = agr.getColor();
			agr.setColor(clrLine);
		}
		// zeichnen lassen
		drawLine(agr);
		// Farbe zuruecksetzen
		if (clrLine != null) {
			agr.setColor(clrTemp);
		}		
		// Nun noch childs (z.B. Tags) haben, die auch gezeichnet
		// werden wollen
		for (int i=0; i < vctChild.size(); i++) {
			((Line)vctChild.elementAt(i)).draw(agr);
		}	
	}
	
	// Farbe setzen
	public void setColor(Color clrNew) { this.clrLine = clrNew; }
	
	public Color getColor() { return this.clrLine; }
	
	// Hier kann man dem Graph weitere Objekte vom Typ Line hinzufuegen
	// wie z.B. Tags. Birgt die Gefahr einer Endlosschleife, z.B:
	// Graph A = new Graph(...), B = new Graph(...);
	// A.addChild(B);
	// B.addChild(A);
	// A.draw(agr);
	// ... und nun sollten wir ein Problem haben
	// ... und deshalb werden die hinzufuegbaren Objekte hiermit auf Tags beschraenkt
	public void addChild(Tag l) {
		vctChild.addElement(l);
	}
	
	public void removeElement(Line l) {
		vctChild.removeElement(l);
	}
	
	// minimales Element in einem Array finden
	protected double getMin(double[] d) {
		double min = d[0];
		for (int i=1; i < d.length; i++) {
			if (d[i] < min)
				min = d[i];
		}
		return min;
	}

	// maximales Element in einem Array finden
	protected double getMax(double[] d) {
		double max = d[0];
		for (int i=1; i < d.length; i++) {
			if (d[i] > max)
				max = d[i];
		}
		return max;
	}

}
