/*
Copyright (C) 2003 Bj\"orn Hoffmeister

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package de.uni_luebeck.tcs.visualization;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import javax.swing.JPanel;

public class Axes extends JPanel implements 
// public class Axes extends Component implements 
	ComponentListener, MouseListener, MouseMotionListener {
// public Konstanten
// X-Achse-Ausrichtung
public static final int TOP = 2;
public static final int BOTTOM = 0;
// Y-Achse-Ausrichtung
public static final int LEFT = 1;
public static final int RIGHT = 3;
// drei Arten der Festlegung der Indexe
public static final int AUTO = 0;
public static final int SEMI_MANUAL = 1;
public static final int MANUAL = 2;

// private Konstanten
// Breite eines Zahlenstrahl in Pixel
private static final int THICK = 1;
// Laaenge einer Markierung auf der Achse
private static final int MARKER = 5;
// allgemeiner Abstand in Pixeln
private static final int SPACE = 3;
// Farben
private static final Color GRID_COLOR = Color.lightGray;

// Groesse des virtuellen Fensters
private double dblX1, dblY1, dblX2, dblY2;
private double dblWidth, dblHeight;

// Groesse und Position des reellen Fensters
private Rectangle rctAxes;

// Vektor der in der Achse enthaltenen Linienobjekte
private Vector lines = new Vector(3);

// Farben der Linien
private Color[] LineColor = {
	Color.blue, Color.green, Color.red, 
	Color.black, Color.cyan, Color.magenta, 
	Color.pink, Color.orange, Color.yellow
	};

// Positionen und Groesse der einzelnen Elemente der Achse
private Rectangle rctTitle;
private Rectangle rctXLabel;
// private Rectangle rctYLabel;
private Rectangle[] rctYLabel;
private Rectangle[] rctXIndex;
private Rectangle[] rctYIndex;
private Rectangle[] rctXMarker;
private Rectangle[] rctYMarker;

// Konstanten und Variablen fuer den Paint-Modus
private static final int PAINT = 0;
private static final int RECALC = 1;
private int flgPaint = RECALC;

// Konstanten und Variablen fuer die Zoomfunktion
private static final double DEFAULT_ZOOM = 2.0;
private static final double MAX_ZOOM = 1024.0;
private double dblZoomFactor = 1.0;
private double dblZoomX1, dblZoomY1, dblZoomX2, dblZoomY2;

// weitere interne Variablen
private char[] chrYLabelArray = null;
// private Image imgYLabel = null;
private double[] dblXIndex = null;
private double[] dblYIndex = null;
private String[] strXIndex = null;
private String[] strYIndex = null;
private Image imgBuffer = null;
private int intPreferredWidth;
private int intPreferredHeight;

// Variablen die vom Benutzer direkt 
// (also normalerweise ueber eine set-Methode)
// geaendert werden koennen
private int ordXLabelPosition = BOTTOM;
private int ordYLabelPosition = LEFT;
private String strTitle = null;
private Font fntTitle = null;
private Color clrTitle = null;
private String strXLabel = null;
private String strYLabel = null;
private double[] dblXIndexManual = null;
private double[] dblYIndexManual = null;
private String[] strXIndexManual = null;
private String[] strYIndexManual = null;
private int intXAuto = AUTO;
private int intYAuto = AUTO;
private Color clrAxesBackground = Color.white;
private boolean blnGrid = false;

//nur zu debug-zwecken
AxesGraphics agrAxes = null;



// Konstruktoren
	public Axes() {
		this(480, 480);
	}

	public Axes(int intPreferredWidth, int intPreferredHeight) {
		super();
		this.intPreferredWidth = intPreferredWidth;
		this.intPreferredHeight = intPreferredHeight;
		this.setFont(new Font("SansSerif", Font.PLAIN, 12));
		flgPaint = this.RECALC;
		
		// Listener hinzufuegen
		this.addComponentListener(this);
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
	}


// Methoden zum zoomen
	// Fuehrt einen Zoom um den angegebenen Faktor durch mit den umgerechneten
	// Punkten x und y als Mittelpunkt, wobei aber nicht gegenueber dem Ausgangswert 
	// vergroessert werden kann.
	// Um auf den Ausgangswert zurueckzukommen, uebergibt man als Zoomfaktor
	// einfach 0.0.
	private void setZoom(int x, int y, double dblFactor) {
		dblZoomFactor *= dblFactor;
		if (dblZoomFactor <= 1.0) {
			dblZoomFactor = 1.0;
		} else {
			if (dblZoomFactor > MAX_ZOOM)
				dblZoomFactor = MAX_ZOOM;
			double dblZoomWidth = dblWidth / dblFactor;
			double dblZoomHeight = dblHeight / dblFactor;
			double dblXY[] = real2virtual(new Point(x, y));
			dblZoomX1 = dblXY[0] - dblZoomWidth / 2;
			if (dblZoomX1 < dblX1)
				dblZoomX1 = dblX1;
			dblZoomX2 = dblZoomX1 + dblZoomWidth;
			if (dblZoomX2 > dblX2) {
				dblZoomX2 = dblX2;
				dblZoomX1 = dblZoomX2 - dblZoomWidth;
			}
			dblZoomY1 = dblXY[1] - dblZoomHeight / 2;
			if (dblZoomY1 < dblY1)
				dblZoomY1 = dblY1;
			dblZoomY2 = dblZoomY1 + dblZoomHeight;
			if (dblZoomY2 > dblY2) {
				dblZoomY2 = dblY2;
				dblZoomY1 = dblZoomY2 - dblZoomHeight;
			}
		}
	}			



// Methoden zur dynamischen Berechnung des Aussehens
	// Rechnet einen Punkt der virtuellen Achse in eine Koordinate im
	// Canvas um
	private Point virtual2real(double dblX, double dblY) {
		return new Point(
			(new java.lang.Long(Math.round(rctAxes.x + 
				((dblX - dblX1) * (rctAxes.width / dblWidth))))).intValue(),
			(new java.lang.Long(Math.round(rctAxes.y + rctAxes.height - 
				((dblY - dblY1) * (rctAxes.height / dblHeight))))).intValue()
		);
	}



	// Rechnet eine Koordinate im Canvas in einen Punkt in der virtuellen
	// Achse um
	private double[] real2virtual(Point pnt) {
		double[] dblXY = new double[2];
		dblXY[0] = (pnt.x - rctAxes.x ) * 
			(dblWidth / rctAxes.width) + dblX1;
		dblXY[1] = (rctAxes.height - (pnt.y - rctAxes.y)) *
			(dblHeight / rctAxes.height) + dblY1;
		return dblXY;
	}		



	// Berechnet eine (hoffentlich) sinnvolle Skalierung fuer eine Achse
	// Parameter:
	// - s, kleinster Wert, der auf der Achse angezeigt wird
	// - e, groesster Wert, der auf der Achse angezeigt wird
	// - n, Anzahl der Intervalle auf der Achse

	/*
	Algorithmus in Matlab
	d = abs(e - s);
	I_pre = d / n;
	I_inc = 10^(floor(log10(I_pre)));
	% Sonderfall beachten
	if ((I_inc == I_pre) & (not(mod(s, I_inc) == 0)))
		I_inc = I_inc / 10;
	end
	% Initialwerte berechnen
	I_first = (floor(I_pre / I_inc));
	I_init = I_first * I_inc;
   	if (mod(s, I_inc) == 0);
      		A = s;
   	else
      		A = floor(s / I_inc) * I_inc;
   	end
   	% Werte optimieren
   	while (A + n * I_init < e)
      		I_init = I_init + I_inc;
   	end
	while (A + n * I_init >= e)
   		n = n - 1;
   	end
   	n = n + 1;
   	% Skala erstellen
   	scale = ((0:n) .* I_init)  + A
	*/

// Hat noch leichte Fehler
	private double[] calcScale(double s, double e, int n) {
		double s_new = s;
		double d = Math.abs(s - e);
		double I_pre = d / n;
		double I_inc = 
			pow10((new java.lang.Double(Math.floor(log10(I_pre)))).longValue());

		// Sonderfall beachten
		if ((I_pre == I_inc) && (s % I_inc != 0.0))
			I_inc /= 10.0;

		// Initialwerte berechnen
		double I_first = Math.round(Math.floor(I_pre / I_inc));
		double I_init =  I_first * I_inc;
		if (s % I_inc != 0.0)
			s_new = Math.floor(s / I_inc) * I_inc;

		// Werte optimieren
		while (s_new + n * I_init < e) 
			I_init += I_inc;
		while (s_new + (n-1) * I_init >= e) 
			n--;

		// Skala erstellen und Werte ggbf. runden
		double I_digits = 
			pow10((new java.lang.Double(Math.floor(log10(I_init)))).longValue());
		double[] scale = new double[n+1];
		scale[0] = s_new;
		for (int i=1; i <= n; i++) {
			scale[i] = scale[i-1] + I_init;
			scale[i] = Math.round(scale[i] / I_digits) * I_digits;
		}

		return scale;
	}



	// Berechnet eine 10er-Potenz
	private double pow10(long e) {
		double x = 1.0;
		if (e < 0) {
			e = Math.abs(e);
			for (int i=0; i < e; i++)
				x = x / 10;
		} else {
			for (int i=0; i < e; i++)
				x = x * 10;
		}
		return x;
	}
	


	// Berechnet eine Darstellung uebergebenen Zahlen die 
	// maximal der Groesse von -888.888 oder -8e-888 entspricht
	private String[] calcScaleLabels(double[] scale) {
		double MAXGO = 1000000.0;
		double MINLO = 0.001;
		double dblAbs, c, e;
		String strC, strE;
		String[] strScale = new String[scale.length];
		
		for (int i=0; i < scale.length; i++) {
			dblAbs = Math.abs(scale[i]);
			// In diesem Fall wird eine Darstellung der Art -8e-888 gewaehlt
			if ((dblAbs >= MAXGO) || (dblAbs < MINLO)) {
				e = Math.floor(log10(dblAbs));
				c = Math.floor(scale[i] / Math.pow(10.0, e));
				strC = Integer.toString((new java.lang.Double(c)).intValue());
				strE = Integer.toString((new java.lang.Double(e)).intValue());
				if (strE.length() > 3)
					strE = strE.substring(0, 4);
				if (strC.equals("0")) {
					strScale[i] = "0";
				} else {
					strScale[i] = strC + "e" + strE;
				}
			// Darstellung in der Form -888.888
			} else {
				strScale[i] = java.lang.Double.toString(scale[i]);
				if (strScale[i].length() > 8)
					strScale[i] = strScale[i].substring(0, 8);
				while (strScale[i].charAt(strScale[i].length()-1) == '0')
					strScale[i] = 
						strScale[i].substring(0, strScale[i].length()-1);
				if (strScale[i].charAt(strScale[i].length()-1) == '.')
					strScale[i] = 
						strScale[i].substring(0, strScale[i].length()-1);
			}
		}
		return strScale;
	}



	// Berechnet den Logarithmus zur Basis 10
	private double log10(double x) {
		return (Math.log(x) / Math.log(10.0));
	}



	// Errechnet eine Skalierung fuer die Achsen und erstellt die entsprechenden
	// Indexe und deren Bezeichner
	private void setScale(int intNumberOfXIndex, int intNumberOfYIndex) {
	// Skalierung der X-Achse berechnen
		// Automatische Skalierung vornehmen
		if (dblZoomFactor > 1.0) {
			dblXIndex = calcScale(dblZoomX1, dblZoomX2, intNumberOfXIndex);
		} else if (intXAuto == AUTO) {
			// Minimum und Maximum der Line-Objekte bestimmen
			double dblMinX, dblMaxX, dblTempMinX, dblTempMaxX;
			if (lines.isEmpty()) {
				dblMinX = 0; dblMaxX = 1;
			} else {
				dblMinX = ((Line)lines.elementAt(0)).getMinX();
				dblMaxX = ((Line)lines.elementAt(0)).getMaxX();				
				for (int i=1; i < lines.size(); i++) {
					dblTempMinX = ((Line)lines.elementAt(i)).getMinX();
					if (dblTempMinX < dblMinX) dblMinX = dblTempMinX;
					dblTempMaxX = ((Line)lines.elementAt(i)).getMaxX();
					if (dblTempMaxX > dblMaxX) dblMaxX = dblTempMaxX;
				}
				if (dblMinX == dblMaxX) {
					dblMinX -= 0.5;
					dblMaxX += 0.5;
				}				
			}
			// ein Array mit den gewuenschten Werten berechnen lassen
			dblXIndex = calcScale(dblMinX, dblMaxX, intNumberOfXIndex);
		// Vorgegebene Werte verwenden
		} else {
			dblXIndex = dblXIndexManual;
		}
		// Groesse des virtuellen Fensters setzen
		dblX1 = dblXIndex[0];
		dblX2 = dblXIndex[dblXIndex.length-1];
		dblWidth = dblX2 - dblX1;
		
	// Skalierung der Y-Achse berechnen
		// Automatische Skalierung vornehmen
		if (dblZoomFactor > 1.0) {
			dblYIndex = calcScale(dblZoomY1, dblZoomY2, intNumberOfYIndex);
		} else if (intYAuto == AUTO) {
			// Minimum und Maximum der Line-Objekte bestimmen
			double dblMinY, dblMaxY, dblTempMinY, dblTempMaxY;
			if (lines.isEmpty()) {
				dblMinY = 0; dblMaxY = 1;
			} else {
				dblMinY = ((Line)lines.elementAt(0)).getMinY();
				dblMaxY = ((Line)lines.elementAt(0)).getMaxY();				
				for (int i=1; i < lines.size(); i++) {
					dblTempMinY = ((Line)lines.elementAt(i)).getMinY();
					if (dblTempMinY < dblMinY) dblMinY = dblTempMinY;
					dblTempMaxY = ((Line)lines.elementAt(i)).getMaxY();
					if (dblTempMaxY > dblMaxY) dblMaxY = dblTempMaxY;
				}
				if (dblMinY == dblMaxY) {
					dblMinY -= 0.5;
					dblMaxY += 0.5;
				}				
			}
			// ein Array mit den gewuenschten Werten berechnen lassen
// System.out.println("Max Y = " + dblMaxY);
			dblYIndex = calcScale(dblMinY, dblMaxY, intNumberOfYIndex);
// System.out.println("Max ScalY = " + dblYIndex[dblYIndex.length - 1]);
		// Vorgegebene Werte verwenden
		} else {
			dblYIndex = dblYIndexManual;
		}
		// Groesse des virtuellen Fensters setzen
		dblY1 = dblYIndex[0];
		dblY2 = dblYIndex[dblYIndex.length-1];
		dblHeight = dblY2 - dblY1;

	// Beschriftung der X-Achse
		// Automatische Generierung der Beschriftung
		if ((intXAuto != MANUAL) || (dblZoomFactor > 1.0)) {
			strXIndex = calcScaleLabels(dblXIndex);
		} else {
			strXIndex = strXIndexManual;
		}
		
	// Beschriftung der Y-Achse
		// Automatische Generierung der Beschriftung
		if ((intYAuto != MANUAL) || (dblZoomFactor > 1.0)) {
			strYIndex = calcScaleLabels(dblYIndex);
		} else {
			strYIndex = strYIndexManual;
		}
	}



	// Berechnet die Position und die Groesse der verschiedenen Elemente,
	// die die graphische Oberflaeche des Axes-Objekts ausmachen
	private void calcLayout(Graphics g) throws AWTException {
	// interne Variablen zum speichern von Abstaenden
		int intTextHeight;
		int intHalfTextHeight;
		int intTitleHeight;
		int intHeightToDisplay;
		int intCharWidth;
		int intIndexWidth;
		int intHalfIndexWidth;
		int intTitleTextHeight;
		int intTitleWidth;
		int intXLabelWidth;
		int intXLabelHeight;
		int intYLabelWidth;
		
	// Objekt zur Berechnung der Abmessungen der Schriftgroesse
		FontMetrics fm = g.getFontMetrics();
		FontMetrics fmTitle = 
			(fntTitle == null)?fm:g.getFontMetrics(fntTitle);
			
	// AxesGraphics-Objekt
		AxesGraphics agrAxes;
		
	// Groesse des Ausgabefensters
		Dimension size = this.getSize();
		intTextHeight = fm.getHeight();
		intHalfTextHeight = (new java.lang.Double(Math.floor(intTextHeight / 2))).intValue();
		intTitleTextHeight = intTextHeight;
		intCharWidth = fm.charWidth('W');
		intIndexWidth = fm.stringWidth("88888888");
		intHalfIndexWidth = (new java.lang.Double(Math.floor(intIndexWidth / 2))).intValue();

		intTitleHeight = (strTitle == null)?0:SPACE + fmTitle.getHeight();
		intXLabelHeight = (strXLabel == null)?0:intTextHeight + SPACE;
		intYLabelWidth = (strYLabel == null)?0:intCharWidth + SPACE;

	// Berechne Groesse und Position der Achsen
		rctAxes = new Rectangle();
		
		rctAxes.x = SPACE;
		switch (ordYLabelPosition) {
		case (RIGHT):
			rctAxes.x += intHalfIndexWidth;
			break;
		default:
			rctAxes.x += intYLabelWidth + intIndexWidth + SPACE;
			break;
		}
		rctAxes.width = size.width;
		rctAxes.width -= (3 * SPACE + intIndexWidth + intHalfIndexWidth + intYLabelWidth);
		if (rctAxes.width < (intIndexWidth + SPACE)) 
			throw new AWTException("Out of Bounds");
		
		rctAxes.y = SPACE + intHalfTextHeight;
		rctAxes.y += intTitleHeight;
		switch (ordXLabelPosition) {
		case (TOP):
			rctAxes.y += intXLabelHeight + intTextHeight + SPACE;
			break;
		default:
			break;
		}
		rctAxes.height = size.height - (3*SPACE + 2*intHalfTextHeight + intTextHeight);
		rctAxes.height -= (intTitleHeight + intXLabelHeight);
		if (rctAxes.height < (intTextHeight + SPACE)) 
			throw new AWTException("Out of Bounds");

	// Berechne IndexArrays und Abmessungen des virtuellen Fensters
		// Vorgabe fuer die Anzahl der Indexe
		int intNumberOfXIndex = 
			(new java.lang.Double(Math.floor((rctAxes.width) / 
			(intIndexWidth + SPACE)) + 1)).intValue();
		int intNumberOfYIndex = 
			Math.max(
				2,
				(new java.lang.Double(Math.floor((rctAxes.height) / 
				(2*intTextHeight + SPACE)) + 1)).intValue()
			);
		setScale(intNumberOfXIndex, intNumberOfYIndex);

	// Berechne Position und Abmessungen der X-Indexe und X-Marker
		int intXIndexX, intXIndexY;
		int intXIndexWidth;
		switch (ordXLabelPosition) {
		case (TOP):
			intXIndexY = rctAxes.y - intHalfTextHeight - SPACE;
			break;
		default:
			intXIndexY = rctAxes.y + rctAxes.height + intHalfTextHeight + SPACE
				+ intTextHeight;
			break;
		}
		rctXIndex = new Rectangle[strXIndex.length];
		rctXMarker = new Rectangle[strXIndex.length];
		for (int i=0; i < strXIndex.length; i++) {
			intXIndexWidth = fm.stringWidth(strXIndex[i]);
			intXIndexX = (virtual2real(dblXIndex[i], 0.0)).x;
			rctXIndex[i] = new Rectangle(
				(new java.lang.Double(intXIndexX - 
					Math.floor(intXIndexWidth / 2))).intValue(),
				intXIndexY,
				intXIndexWidth,
				intTextHeight
			);
			rctXMarker[i] = new Rectangle(
				intXIndexX,
				-1,			// -1 bedeutet, dass die Marker sowohl
							// am Kopf als auch am Fuss ausgegeben
							// werden
				THICK,
				MARKER
			);
		}
		
	// Berechne Position und Abmessung der Y-Indexe und Y-Marker
		int intYIndexX, intYIndexY;
		int intYIndexWidth;
		boolean blnXOffset = true;
		switch (ordYLabelPosition) {
		case (RIGHT):
			intYIndexX = rctAxes.x + rctAxes.width + SPACE;
			blnXOffset = false;
			break;
		default:
			intYIndexX = rctAxes.x - (intIndexWidth + SPACE);
			break;
		}
		rctYIndex = new Rectangle[strYIndex.length];			
		rctYMarker = new Rectangle[strYIndex.length];			
		for (int i=0; i < strYIndex.length; i++) {
			intYIndexWidth = fm.stringWidth(strYIndex[i]);
			intYIndexY = (virtual2real(0.0, dblYIndex[i])).y;
			rctYIndex[i] = new Rectangle(
				intYIndexX + ((blnXOffset)?(intIndexWidth - intYIndexWidth):0),
				intYIndexY + intHalfTextHeight 
					- (fm.getLeading() + fm.getDescent()),
				intYIndexWidth,
				intTextHeight
			);
			rctYMarker[i] = new Rectangle(
				-1,			// -1 bedeutet, dass die Marker sowohl
							// links als auch rechts ausgegeben
							// werden
				intYIndexY,
				MARKER,
				THICK
			);
		}

	// Berechne Position und Abmessung des Titels
		if (this.strTitle == null)
			rctTitle = null;
		else {
			rctTitle = new Rectangle(
				Math.min(
					size.width - 2*SPACE, 
					fmTitle.stringWidth(this.strTitle)
				),
				intTitleHeight
			);
			rctTitle.x = (new java.lang.Double(Math.floor((size.width 
				- rctTitle.width) / 2))).intValue();
			rctTitle.y = SPACE + intTitleHeight;
		}
		
	// Berechne Position und Abmessung des XLabels
		if (this.strXLabel == null)
			rctXLabel = null;
		else {
			rctXLabel = new Rectangle(
				Math.min(rctAxes.width, fm.stringWidth(this.strXLabel)),
				intTextHeight
			);
			rctXLabel.x = rctAxes.x 
				+ (new java.lang.Double(Math.floor((rctAxes.width 
				- rctXLabel.width) / 2))).intValue();
			switch (ordXLabelPosition) {
			case (TOP):
				rctXLabel.y = intTitleHeight + SPACE + intTextHeight;
				break;
			default:
				rctXLabel.y = rctAxes.y + rctAxes.height
					+ intHalfTextHeight + intTextHeight + 2*SPACE
					+ intTextHeight;
				break;
			}
		}
			
	// Berechne Position und Abmessung des YLabels
/*
		if (this.strYLabel == null)
			rctYLabel = null;
		else {
			rctYLabel = new Rectangle(
				Math.min(rctAxes.height, fm.stringWidth(this.strYLabel)),
				intTextHeight
			);
			switch (ordYLabelPosition) {
			case (RIGHT):
				rctYLabel.x = rctAxes.x + rctAxes.width 
					+ intIndexWidth + 2*SPACE;
				break;
			default:
				rctYLabel.x = SPACE;
				break;
			}
			rctYLabel.y = rctAxes.y + 
				(new java.lang.Double(Math.round((rctAxes.height - 
				rctYLabel.width) / 2))).intValue();
			imgYLabel = this.createImage(rctYLabel.width, rctYLabel.height);
			Graphics grpYLabel = imgYLabel.getGraphics();
			grpYLabel.setFont(this.getFont());
			grpYLabel.setColor(this.getForeground());
			grpYLabel.drawString(strYLabel, 0, rctYLabel.height);
		}
*/			
			
		if (this.strYLabel == null)
			rctYLabel = null;
		else {
			int intMaxYLabelChar = Math.min(
					(new java.lang.Double(Math.floor(rctAxes.height 
					/ intTextHeight))).intValue(),
					strYLabel.length()
				);
			int intYLabelX;
			int intYLabelY = rctAxes.y + (new java.lang.Double(Math.round(
				(rctAxes.height - intMaxYLabelChar * intTextHeight) 
				/ 2))).intValue() - (fm.getLeading() + fm.getDescent());
			switch (ordYLabelPosition) {
			case (RIGHT):
				intYLabelX = rctAxes.x + rctAxes.width 
					+ intIndexWidth + 2*SPACE;
				break;
			default:
				intYLabelX = SPACE;
				break;
			}
			rctYLabel = new Rectangle[intMaxYLabelChar];
			chrYLabelArray = new char[intMaxYLabelChar];
			for (int i=0; i < rctYLabel.length; i++) {
				chrYLabelArray[i] = strYLabel.charAt(i);
				rctYLabel[i] = new Rectangle(
					intYLabelX,
					intYLabelY + i*intTextHeight,
					fm.charWidth(strYLabel.charAt(i)),
					intTextHeight
				);
			}
		}
	}
/*
//	public void repaint(long tm) { repaint(); }

	public void repaint(int x, int y, int width, int height) { repaint(); }

//	public void repaint(long tm, int x, int y, int width, int height) { repaint(); }

	public void repaint() {
//		repaint();
		repaint();
	}

	public void repaint(int flag) {
		flgPaint = flag;
		super.repaint();
	}
*/
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		flgPaint = RECALC;
	// Berechnet die Position der einzelnen Elemente, die Skalierung
	// und den Inhalt des eigentlichen Achsenobjekts neu
		try {
			if (flgPaint == RECALC) {
				calcLayout(g);
				// Laesst die enthaltene Objekte sichselber zeichnen
				// Erzeuge ein AxesGraphicsobjekt
				imgBuffer = this.createImage(rctAxes.width, rctAxes.height);
				Graphics grpBuffer = imgBuffer.getGraphics();
				grpBuffer.setColor(g.getColor());
				grpBuffer.setFont(g.getFont());
//			AxesGraphicsInterface agrAxes = new AxesGraphics(
				agrAxes = new AxesGraphics(
					grpBuffer,
					clrAxesBackground, GRID_COLOR,
					this.getFont(),
					dblX1, dblY1, dblX2, dblY2,
					rctAxes.width, rctAxes.height
				);
				// Setze Hintergrundfarbe
				agrAxes.clear();
				// Zeichne Grid
				if (blnGrid)
					agrAxes.drawGrid(dblXIndex, dblYIndex);
				// Lasse die einzelnen Linienobjekte sich selber zeichnen
				for (int i=0; i < lines.size(); i++) {
					agrAxes.setColor(LineColor[i % LineColor.length]);
					((Line)lines.elementAt(i)).draw(agrAxes);
				}
			}
		
		// Nun beginnt die Ausgabe auf den Bildschirm
			// Das eigentliche Axes-Objekt mit Rahmen
			g.drawImage(imgBuffer, rctAxes.x, rctAxes.y, this);
			g.drawRect(rctAxes.x, rctAxes.y, rctAxes.width, rctAxes.height);
	
			// Die Marker
			for (int i=0; i < rctXMarker.length; i++) {
				if (rctXMarker[i].y == -1) {
					g.fillRect(
						rctXMarker[i].x, 
						rctAxes.y, 
						rctXMarker[i].width, 
						rctXMarker[i].height
					);
					g.fillRect(
						rctXMarker[i].x, 
						rctAxes.y + rctAxes.height - rctXMarker[i].height, 
						rctXMarker[i].width,
						rctXMarker[i].height
					);
				} else {
					g.fillRect(
						rctXMarker[i].x, 
						rctXMarker[i].y, 
						rctXMarker[i].width, 
						rctXMarker[i].height
					);
				}
			}	
			for (int i=0; i < rctYMarker.length; i++) {
				if (rctYMarker[i].x == -1) {
					g.fillRect(
						rctAxes.x,
						rctYMarker[i].y,
						rctYMarker[i].width, 
						rctYMarker[i].height
					);
					g.fillRect(
						rctAxes.x + rctAxes.width - rctYMarker[i].width,
						rctYMarker[i].y,
						rctYMarker[i].width,
						rctYMarker[i].height
					);
				} else {
					g.fillRect(
						rctYMarker[i].x, 
						rctYMarker[i].y, 
						rctYMarker[i].width, 
						rctYMarker[i].height
					);
				}
			}									
	
			// Die Indexe
			for (int i=0; i < rctXIndex.length; i++)
				g.drawString(strXIndex[i], rctXIndex[i].x, rctXIndex[i].y);
			for (int i=0; i < rctYIndex.length; i++)
				g.drawString(strYIndex[i], rctYIndex[i].x, rctYIndex[i].y);		
	
			// Die Labels
			if (strXLabel != null)
				g.drawString(strXLabel, rctXLabel.x, rctXLabel.y);
/*			if (strYLabel != null)
				g.drawImage(imgYLabel, rctYLabel.x, rctYLabel.y, 
					rctYLabel.x + rctYLabel.width, rctYLabel.y + rctYLabel.width,
					rctYLabel.width, rctYLabel.height, 0, 0, this);
*/
			if (strYLabel != null)
				for (int i=0; i < rctYLabel.length; i++) {
					g.drawString(String.valueOf(chrYLabelArray[i]), 
						rctYLabel[i].x, rctYLabel[i].y);
				}
				
			// Der Titel
			if (strTitle != null) {
				Font fntTemp = g.getFont();
				Color clrTemp = g.getColor();
				if (fntTitle != null) 
					g.setFont(fntTitle);
				if (clrTitle != null)
					g.setColor(clrTitle);
				g.drawString(strTitle, rctTitle.x, rctTitle.y);
				if (fntTitle != null) 
					g.setFont(fntTemp);
				if (clrTitle != null)
					g.setColor(clrTemp);
			}
			// Normalerweise sollen die Positionen, Skalierungen und die Ausgabe der 
			// Line-Objektenicht neu berechnet werden
			flgPaint = PAINT;
		}
		catch (AWTException e) {
			// es wird die Fehlermeldung ausgegeben
			FontMetrics fm = g.getFontMetrics();
			Dimension size = getSize(null);
			String str = e.toString().substring(23, e.toString().length());
			g.drawString(
				str,
				Math.round((size.width - fm.stringWidth(str)) / 2),
				Math.round((size.height - fm.getHeight()) / 2) + fm.getHeight()
			);
			flgPaint = RECALC;
		}
	}



// weitere Methoden
	// Gibt true zurueck, wenn sich die uebergebene Koordinate im
	// Achsenobjekt befindet
	private boolean isInAxes(int x, int y) {
		try {
			return (
				(x >= rctAxes.x)
				&& (x <= rctAxes.x + rctAxes.width)
				&& (y >= rctAxes.y)
				&& (y <= rctAxes.y + rctAxes.height)
			);
		}
		catch (Exception e) {
			return false;
		}
	}


	
// Setter- und Getter-Methoden
	// set/get-Methode fuer die Position des X-Labels
	// Parameter: - eine der vorgegebenen Konstanten
	//		unbekannte Eingaben werden ignoriert
	//		und stattdessen werden Standardwerte verwendet
	public void setXLabelPosition(int ordXLabelPosition) {
		if (this.ordXLabelPosition != ordXLabelPosition) {
			this.ordXLabelPosition = ordXLabelPosition;
			repaint();
		}
	}
	public int getXLabelPosition() {
		return this.ordXLabelPosition;
	}
	
	// set/get-Methode fuer die Position des Y-Labels
	// Parameter: - eine der vorgegebenen Konstanten
	//		unbekannte Eingaben werden ignoriert
	//		und stattdessen werden Standardwerte verwendet
	public void setYLabelPosition(int ordYLabelPosition) {
		if (this.ordYLabelPosition != ordYLabelPosition) {
			this.ordYLabelPosition = ordYLabelPosition;
			repaint();
		}
	}
	public int getYLabelPosition() {
		return this.ordYLabelPosition;
	}
	
	// set/get-Methode fuer den Titel, falls null uebergeben wird,
	// wird keine Titel dargestellt
	// Parameter: - String oder null
	public void setTitle(String strTitle) {
		this.strTitle = strTitle;
		repaint();
	}
	public String getTitle() {
		return this.strTitle;
	}
	
	// set/get-Methode fuer den Font (Schriftart, Stil, Schriftgroesse)
	// in dem der Titel dargestellt wird. Falls der Font null ist, wird
	// der Standardfont zur Darstellung verwendet
	// Parameter: - Font oder null
	public void setTitleFont(Font fntTitle) {
		this.fntTitle = fntTitle;
		repaint();
	}
	public Font getTitleFont() {
		return this.fntTitle;
	}
	
	// set/get-Methode fuer die Farbe in der der Titel dargestellt wird. 
	// Falls die Farbe null ist, wird die Standardfarbe zur Darstellung verwendet
	// Parameter: - Color oder null
	public void setTitleColor(Color clrTitle) {
		this.clrTitle = clrTitle;
		repaint();
	}
	public Color getTitleColor() {
		return this.clrTitle;
	}
	
	// set/get-Methode fuer das X-Label, falls null uebergeben wird,
	// wird keine X-Label dargestellt
	// Parameter: - String oder null
	public void setXLabel(String strXLabel) {
		this.strXLabel = strXLabel;
		repaint();
	}
	public String getXLabel() {
		return this.strXLabel;
	}
	
	// set/get-Methode fuer das Y-Label, falls null uebergeben wird,
	// wird keine Y-Label dargestellt
	// Parameter: - String oder null
	public void setYLabel(String strYLabel) {
		this.strYLabel = strYLabel;
		repaint();
	}
	public String getYLabel() {
		return this.strYLabel;
	}
	
	// set/get-Methode fuer die Skalierung der X-Achse
	// es wird automatisch auf die Anzeige der vorgegebenen Werte geschaltet
	// Parameter: -double[]
	public void setXIndex(double[] dblXIndexManual) throws Exception {
		double[] dblXIndexManualTemp = this.dblXIndexManual;
		this.dblXIndexManual = dblXIndexManual;
		int intXAuto = this.SEMI_MANUAL;
		if (this.intXAuto == this.MANUAL)
			intXAuto = this.MANUAL;
		else if (dblXIndexManual == null)
			intXAuto = this.AUTO;
		try {
			setXAuto(intXAuto);
		}
		// Im Fehlerfall alten Zustand wieder herstellen und Fehler ausgeben
		catch (Exception e) {
			this.dblXIndexManual = dblXIndexManualTemp;
			throw e;
		}
	}
	public double[] getXIndex() {
		return this.dblXIndex;
	}
	
	// set/get-Methode fuer die Beschriftung der X-Achse
	// es wird automatisch die Beschriftung angezeigt
	// Parameter: - String[] oder null
	public void setXIndexLabel(String[] strXIndexManual) throws Exception {
		String[] strXIndexManualTemp = this.strXIndexManual;
		this.strXIndexManual = strXIndexManual;
		int intXAuto = this.MANUAL;
		if (strXIndexManual == null)
			intXAuto = this.SEMI_MANUAL;
		try {
			setXAuto(intXAuto);
		}
		// Im Fehlerfall alten Zustand wieder herstellen und Fehler ausgeben
		catch (Exception e) {
			this.strXIndexManual = strXIndexManualTemp;
			throw e;
		}
	}
	public String[] getXIndexLabel() {
		return this.strXIndex;
	}
	
	// set/get-Methode fuer den Anzeigemodus der Skalierung der Y-Achse
	// Parameter: 	AUTO (berechnet Skalierung und Beschriftung selber)
	//		|SEMI_MANUAL (Skalierung ist vorgegeben, aber Beschriftung wird berechnet)
	//		|MANUAL (Skalierung und Beschriftung ist vorgegeben)
	public void setXAuto(int intXAuto) throws Exception {
		switch (intXAuto) {
		case (MANUAL):
			if (this.strXIndexManual == null)
				throw new Exception
					("XIndexLabel must not be null");
			if (this.dblXIndexManual.length != strXIndexManual.length)
				throw new Exception
					("XIndex and XIndexLabel must have the same length");
		case (SEMI_MANUAL):
			if (this.dblXIndexManual == null)
				throw new Exception
					("XIndex must not be null");
			if (this.dblXIndexManual.length < 2)
				throw new Exception
					("XIndex must have at least two entries");
		}
		this.intXAuto = intXAuto;
		repaint();
		
	}
				
	// set/get-Methode fuer die Skalierung der Y-Achse
	// es wird automatisch auf die Anzeige der vorgegebenen Werte geschaltet
	// Parameter: -double[]
	public void setYIndex(double[] dblYIndexManual) throws Exception {
		double[] dblYIndexManualTemp = this.dblYIndexManual;
		this.dblYIndexManual = dblYIndexManual;
		int intYAuto = this.SEMI_MANUAL;
		if (this.intYAuto == this.MANUAL)
			intYAuto = this.MANUAL;
		else if (dblYIndexManual == null)
			intYAuto = this.AUTO;
		try {
			setYAuto(intYAuto);
		}
		// Im Fehlerfall alten Zustand wieder herstellen und Fehler ausgeben
		catch (Exception e) {
			this.dblYIndexManual = dblYIndexManualTemp;
			throw e;
		}
	}
	public double[] getYIndex() {
		return this.dblYIndex;
	}
	
	// set/get-Methode fuer die Beschriftung der Y-Achse
	// es wird automatisch die Beschriftung angezeigt
	// Parameter: - String[] oder null
	public void setYIndexLabel(String[] strYIndexManual) throws Exception {
		String[] strYIndexManualTemp = this.strYIndexManual;
		this.strYIndexManual = strYIndexManual;
		int intYAuto = this.MANUAL;
		if (strYIndexManual == null)
			intYAuto = this.SEMI_MANUAL;
		try {
			setYAuto(intYAuto);
		}
		// Im Fehlerfall alten Zustand wieder herstellen und Fehler ausgeben
		catch (Exception e) {
			this.strYIndexManual = strYIndexManualTemp;
			throw e;
		}
	}
	public String[] getYIndexLabel() {
		return this.strYIndex;
	}
	
	// set/get-Methode fuer den Anzeigemodus der Skalierung der Y-Achse
	// Parameter: 	AUTO (berechnet Skalierung und Beschriftung selber)
	//		|SEMI_MANUAL (Skalierung ist vorgegeben, aber Beschriftung wird berechnet)
	//		|MANUAL (Skalierung und Beschriftung ist vorgegeben)
	public void setYAuto(int intYAuto) throws Exception {
		switch (intYAuto) {
		case (MANUAL):
			if (this.strYIndexManual == null)
				throw new Exception
					("YIndexLabel must not be null");
			if (this.dblYIndexManual.length != strYIndexManual.length)
				throw new Exception
					("YIndex and YIndexLabel must have the same length");
		case (SEMI_MANUAL):
			if (this.dblYIndexManual == null)
				throw new Exception
					("YIndex must not be null");
			if (this.dblYIndexManual.length < 2)
				throw new Exception
					("YIndex must have at least two entries");
		}
		this.intYAuto = intYAuto;
		repaint();		
	}
	
	// set/get-Methode fuer die Hintergrundfarbe des Achsenobjektes
	public void setAxesBackground(Color clrAxesBackground) {
		this.clrAxesBackground = clrAxesBackground;
		repaint();
	}
	public Color getAxesBackground() {
		return this.clrAxesBackground;
	}
	
	// set/get-Methode fuer die Anzeige des Grid
	public void setGrid(boolean blnGrid) {
		this.blnGrid = blnGrid;
		repaint();
	}
	public boolean getGrid() {
		return this.blnGrid;
	}



// Methoden zum hinzufuegen und entfernen von Line-Objekten
	// hinzufuegen
	public void addLine(Line line) {
		lines.addElement(line);
		repaint();
	}
	
	// entfernen
	public void removeLine(Line line) {
		lines.removeElement(line);
		repaint();
	}

	// alle entfernen
	public void removeAllLines() {
		lines.removeAllElements();
		repaint();
	}		

// Listenermethoden
	// ComponentListener-Methoden
	public void componentHidden(ComponentEvent e) {}
	
	public void componentMoved(ComponentEvent e) {}
	
	// In diesem Fall muss vor dem neuzeichnen das Layout neu berechnet werden
	// da repaint() sowieso aufgerufen wird, reicht es das flag zu setzen
	public void componentResized(ComponentEvent e) {
		this.flgPaint = RECALC;
	}
	
	public void componentShown(ComponentEvent e) {}



	// MouseListener
	// Falls der Click im Achsenobjekt erfolgt, dann wird ein Zoom ausgefuehrt
	// linke Maustaste = einzoomen
	// rechte Maustaste = auszoomen
	// rechte Maustaste + Shift = auf Originalskalierung zurueckkehren
	public void mouseClicked(MouseEvent e) {
		if (isInAxes(e.getX(), e.getY())) {
			int intMods = e.getModifiers();
			if ((intMods & InputEvent.BUTTON1_MASK) != 0) {
				setZoom(e.getX(), e.getY(), DEFAULT_ZOOM);
				repaint();
				
			} else if ((intMods & InputEvent.BUTTON3_MASK) != 0) {
				if (e.isShiftDown()) {
					setZoom(e.getX(), e.getY(), 0.0);
				} else {
					setZoom(e.getX(), e.getY(), (1.0 / DEFAULT_ZOOM));
				}
				repaint();
			}
		}
	}

	public void mouseEntered(MouseEvent e) {}
	
	// Stelle sicher, dass beim verlassen der DEFAULT-CURSOR angezeigt wird
	public void mouseExited(MouseEvent e) {
		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}
	
	public void mousePressed(MouseEvent e) {}
	
	public void mouseReleased(MouseEvent e) {}
	
	
		
	// MouseMotionListener
	// Leite Ereignisbearbeitung weiter
	public void mouseDragged(MouseEvent e) {
		mouseMoved(e);
	}
	
	// Zeige CROSSHAIR-CURSOR an, falls sich die Maus ueber dem Achsenobjekt befindet,
	// ansonsten den DEFAULT-CURSOR
	public void mouseMoved(MouseEvent e) {
		if (isInAxes(e.getX(), e.getY())) {
			setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
		} else {
			setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		}
	}



// Gewuenschte Groesse zurueckgeben
/*	public Dimension getMaximumSize() {
		return getPreferredSize();
	}
	
	public Dimension getMinimumSize() {
		return getPreferredSize();
	}
*/	
	public Dimension getPreferredSize() {
		return new Dimension(intPreferredWidth, intPreferredHeight);
	}	

// Debug-Funktionen
	void dbg(String str) {
		System.out.println(str);
	}

}








// Alte Codefragmente...

/*
		
		if (this.size == null) getSize(this.size);
	}
	
	
		int textHeight = fm.getHeight();
		int halfTextHeight = Math.floor(textHeight / 2);
		int textWidth = fm.getWidths();
		int indexWidth = fm.stringWidth("888.888");
		int halfTextWidth = Math.floor(indexWidth / 2);
		
		int titleHeight = (this.titel == null)?0:(SPACE + textHeight)
			+ (SPACE + halfTextHeight);
		int emptyWidth = (halfIndexWidth + SPACE);
		int xHeight = (halfTextHeight + SPACE)
			+ (textHeight + SPACE)
			+ (this.xLabel == null)?0:(textHeight + SPACE);
		int yWidth = (this.yLabel == null)?0:(SPACE + textWidth)
			+ (SPACE + indexWidth)
			+ SPACE;

		// Berechne reale Ausgabefeldgroesse 
		int realX = (this.positionY == LEFT)?(yWidth):(emptyWidth);
		int realY = titleHeight + (this.positionX == BOTTOM)?0:
			(xHeight - (halfTextHeight + SPACE));
		int realWidth = this.size.width - (yWidth + emptyWidth);
		int realHeight = this.size.height - (titleHeight + xHeight);
		
		// Berechne Indexe 
		real[] indexX = getIndexX(realWidth + indexWidth);
		real[] indexY = getIndexY(realHeight + textHeight);
		
		// Berechne virtuelle Ausgabefeldgroesse 
		real virtualX = indexX[0];
		real virtualY = indexY[0];
		real virtualWidth = indexX[indexX.length-1] - indexX[0];
		real virtualHeight = indexY[indexY.length-1] - indexY[0];
		

// Graphen, die in diesem Achsenobjekt angezeigt werden
// private Graph[] gphChild = null;


/*
	Dimension size = null;
	Virtual2Real v2r = null;

	String title = null;

	int positionX = BOTTOM;
	String labelX = null;
	long[] indexX = null;
	boolean autoX = true;
	
	int positionY = LEFT;
	String labelY = null;
	long[] indexY = null;
	boolean autoY = true;
	
	class Virtual2Real {
		Virtual2Real v2r = new Virtual2Real(realX, realY, realWidth, realHeight,
					virtualX, virtualY, virtualWidth, virtualHeight);

		
		public Virtual2Real(realX, realY, realWidth, realHeight,
					virtualX, virtualY, virtualWidth, virtualHeight);
this.g = g;
			this.realDimension = realDimension;
			this.virtualDimension = virtualDimension;
			
		}
		
		public Point getRealPoint(Point point) {}
		
		public long getRealX(long x) {}
		
		public long getRealY(long y) {}
		
	}
	
	public void setPositionX(int position) { this.positionX = position; }
	public int getPositionX() { return this.positionX; }
	
	public void setLabelX(String label) { this.labelX = Label; }
	public String getLabelX() { return this.labelX; }
	
	public void setIndexX(long[] index) { this.indexX = index; }
	public long[] getIndexX() { return this.indexX; }
	
	public void setAutoX(boolean auto) { this.autoX = auto; }
	public void setAutoX(boolean auto) { this.autoX = auto; }

	public void setPositionY(int position) { this.positionY = position; }
	public int getPositionY() { return this.positionY; }
	
	public void setLabelY(String label) { this.labelY = Label; }
	public String getLabelY() { return this.labelY; }
	
	public void setIndexY(long[] indeY) { this.indexY = indeY; }
	public long[] getIndexY() { return this.indexY; }
	
	public void setAutoY(boolean auto) { this.autoY = auto; }
	public void setAutoY(boolean auto) { this.autoY = auto; }
	
	public void paint(Graphics g) {
		FontMetrice fm = g.getFontMetrics();
		if (this.size == null) getSize(this.size);

		int textHeight = fm.getHeight();
		int halfTextHeight = Math.floor(textHeight / 2);
		int textWidth = fm.getWidths();
		int indexWidth = fm.stringWidth("888.888");
		int halfTextWidth = Math.floor(indexWidth / 2);
		
		int titleHeight = (this.titel == null)?0:(SPACE + textHeight)
			+ (SPACE + halfTextHeight);
		int emptyWidth = (halfIndexWidth + SPACE);
		int xHeight = (halfTextHeight + SPACE)
			+ (textHeight + SPACE)
			+ (this.xLabel == null)?0:(textHeight + SPACE);
		int yWidth = (this.yLabel == null)?0:(SPACE + textWidth)
			+ (SPACE + indexWidth)
			+ SPACE;

		// Berechne reale Ausgabefeldgroesse 
		int realX = (this.positionY == LEFT)?(yWidth):(emptyWidth);
		int realY = titleHeight + (this.positionX == BOTTOM)?0:
			(xHeight - (halfTextHeight + SPACE));
		int realWidth = this.size.width - (yWidth + emptyWidth);
		int realHeight = this.size.height - (titleHeight + xHeight);
		
		// Berechne Indexe 
		real[] indexX = getIndexX(realWidth + indexWidth);
		real[] indexY = getIndexY(realHeight + textHeight);
		
		// Berechne virtuelle Ausgabefeldgroesse 
		real virtualX = indexX[0];
		real virtualY = indexY[0];
		real virtualWidth = indexX[indexX.length-1] - indexX[0];
		real virtualHeight = indexY[indexY.length-1] - indexY[0];
		
		// Erstelle virtual2real-Objekt  
		Virtual2Real v2r = new Virtual2Real(realX, realY, realWidth, realHeight,
					virtualX, virtualY, virtualWidth, virtualHeight);
		
				
		
		
		
	public void componentHidden(ComponentEvent e) {}	
	public void componentMoved(ComponentEvent e) {}
	public void componentResized(ComponentEvent e) {
		getSize(this size);
		repaint();
	}
	public void componentShown(ComponentEvent e) {}
// ZU UEBERARBEITEN

	// Berechne Position und Abmessung der X-Indexe
		int intXIndexWidth = Math.floor((rctAxes.width + 2*intHalfIndexWidth) 
					/ strXIndex.length);
		int intXOffset;
		int intXIndexY, intXMarkerY;
		switch (ordXLabelPosition) {
		case (this.TOP):
			intXIndexY = rctAxes.y - (intTextHeight + SPACE);
			intXMarkerY = rctAxes.y;
			break;
		default:
			intXIndexY = rctAxes.y + rctAxes.height + SPACE;
			intXMarkerY = rctAxes.y + rctAxes.height - MARKER;
			break;
		}
		rctXIndex = new Rectangle[strXIndex.length];			
		for (int i=0; i < strXIndex.length; i++) {
			rctXIndex[i] = new Rectangle(
				fm.StringWidth(strXIndex[i]),
				intTextHeight
			);
			int intXOffset = 0;
			if (intXIndexWidth != rctXIndex[i].width)
				intXOffset = Math.floor((intXIndexWidth - rctXIndex[i].width) 
						/ 2);
			rctXIndex[i].x = rctAxes.x - intHalfIndexWidth + intXOffset;
			rctXIndex[i].y = intXIndexY;
		}
		
	// Berechne Position und Abmessung der X-Marker
		rctXMarker = new Rectangle[strXIndex.length];
		for (int i=0; i < strXIndex.length; i++)
			rctXMarker[i] = new Rectangel(
				rctAxes.x + i*intXIndexWidth,
				intXMarkerY,
				THICK,
				MARKER
			);
		rctXMarker[strXIndex-1].x = rctAxes.x + rctAxes.width;
		
	// Berechne Position und Abmessung der Y-Indexe
		int intYIndexHeight = Math.floor((rctAxes.height + 2*intHalfTextHeight) 
					/ strYIndex.length);
		int intYIndexX, intYMarkerX;
		switch (ordYLabelPosition) {
		case (this.RIGHT):
			intYIndexX = rctAxes.x + rctAxes.width + SPACE;
			intYMarkerX = rctAxes.x + rctAxes.width - MARKER;
			intXOffset = -1;
			break;
		default:
			intYIndexX = rctAxes.x + (intIndexWidth + SPACE);
			intYMarkerX = rctAxes.x;
			break;
		rctYIndex = new Rectangle[strYIndex.length];
		for (int i=0; i < strYIndex.length; i++) {
			rctYIndex[i] = new Rectangle(
				fm.stringWidth(strYIndex[i]),
				intTextHeight
			);
			intXOffset = (ordYLabelPosition == this.RIGHT)?
				0:(intIndexWidth - rctYIndex[i].width);
			rctYIndex[i].x = intYIndexX + intXOffset;
			rctYIndex[i].y = rctAxes.y - halfTextHeight + i*intYIndexHeight;	
		}
		
	// Berechne Position und Abmessung der Y-Marker
		rctYMarker = new Rectangle[strYIndex.length];
		for (int i=0; i < strYIndex.length; i++)
			rctXMarker[i] = new Rectangel(
				intYMarkerX,
				rctAxes.y + i*intYIndexHeight,
				MARKER,
				THICK
			);

// ENDE UEBERARBEITEN

*/
