/*
Copyright (C) 2003 Bj\"orn Hoffmeister

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package de.uni_luebeck.tcs.visualization;

import java.awt.*;

public class AxesGraphics implements AxesGraphicsInterface {
	Graphics g;
	Font f;
	Color clrBackground, clrGrid;
	double dblX1, dblX2, dblY1, dblY2;
	int intWidth, intHeight;
	double dblXFactor, dblYFactor;

	public AxesGraphics(
		Graphics g, 
		Color clrBackground, Color clrGrid,
		Font f,
		double dblX1, double dblY1, double dblX2, double dblY2, 
		int intWidth, int intHeight
	) {
		this.g = g;
		this.clrBackground = clrBackground;
		this.clrGrid = clrGrid;
		this.f = f;
		this.dblX1 = dblX1;
		this.dblY1 = dblY1;
		this.dblX2 = dblX2;
		this.dblY2 = dblY2;
		this.intWidth = intWidth;
		this.intHeight = intHeight;
		this.dblXFactor = intWidth / (dblX2 - dblX1);
		this.dblYFactor = intHeight / (dblY2 - dblY1);
	}
	
	public void clear() {
		Color clrTemp = g.getColor();
		g.setColor(clrBackground);
		g.fillRect(0, 0, intWidth, intHeight);
		g.setColor(clrTemp);
	}
	
	public void drawGrid(double[] dblX, double[] dblY) {
		Point pnt;
		Color clrTemp = g.getColor();
		g.setColor(clrGrid);
		for (int i=0; i < dblX.length; i++) {
			pnt = virtual2real(dblX[i], 0.0);
			g.drawLine(pnt.x, 0, pnt.x, intHeight);
		}
		for (int i=0; i < dblY.length; i++) {
			pnt = virtual2real(0.0, dblY[i]);
			g.drawLine(0, pnt.y, intWidth, pnt.y);
		}
		g.setColor(clrTemp);
	}	
	
	public void drawPoint(double x, double y) {
		Point pnt = virtual2real(x, y);
		g.drawLine(pnt.x, pnt.y, pnt.x, pnt.y);
	}

	public void drawPoint(double x, double y, char c) {
		if (c == '.') {
			drawPoint(x, y);
		} else {
			Point pnt = virtual2real(x, y);
			FontMetrics fm = g.getFontMetrics();
			int intXOffset = Math.round(fm.charWidth(c) / 2);
			int intYOffset = Math.round(fm.getHeight() / 2);
			g.drawString(
				String.valueOf(c), 
				pnt.x - intXOffset, 
				pnt.y - intYOffset + fm.getHeight() 
					- (fm.getLeading() + fm.getDescent())
			);
		}
	}
	
	public void drawLine(double x1, double y1, double x2, double y2) {
		Point pnt1 = virtual2real(x1, y1);
		Point pnt2 = virtual2real(x2, y2);
		g.drawLine(pnt1.x, pnt1.y, pnt2.x, pnt2.y);
	}
	
	public void drawLine(double[] dblX, double[] dblY) {
		int n = dblX.length;
		int[] intX = new int[n];
		int[] intY = new int[n];
		Point pnt;
		for (int i=0; i < n; i++) {
			pnt = virtual2real(dblX[i], dblY[i]);
			intX[i] = pnt.x;
			intY[i] = pnt.y;
		}
		g.drawPolyline(intX, intY, n);
	}

	public void fillCircle(double dblX, double dblY, int d) {
	    Point pnt = virtual2real(dblX, dblY);
	    pnt.x -= d/2;
	    pnt.y -= d/2;
	    g.fillOval(pnt.x, pnt.y, d, d);
	}	

	public void fillLine(double[] dblX, double[] dblY) {
		int n = dblX.length;
		int[] intX = new int[n];
		int[] intY = new int[n];
		Point pnt;
		for (int i=0; i < n; i++) {
			pnt = virtual2real(dblX[i], dblY[i]);
			intX[i] = pnt.x;
			intY[i] = pnt.y;
		}
		g.fillPolygon(intX, intY, n);
	}
	
	public void fillRect(double dblX, double dblY, double dblWidth, double dblHeight) {
	    Point pnt0 = virtual2real(dblX, dblY);
	    Point pnt1 = virtual2real(dblX + dblWidth, dblY + dblHeight);
	    g.fillRect(pnt0.x, pnt0.y, pnt1.x - pnt0.x, pnt1.y - pnt0.y);
	}

	public void drawString(double x, double y, String str) {
		int OFFSET = 4;
		FontMetrics fm = g.getFontMetrics();
		Point pnt = virtual2real(x, y);
		Rectangle rct = new Rectangle(fm.stringWidth(str), fm.getHeight());
		rct.x = pnt.x + OFFSET;
		rct.y = pnt.y + Math.round(fm.getHeight() / 2) 
			- (fm.getLeading() + fm.getDescent());
		if (rct.x + rct.width > intWidth) {
			rct.x = pnt.x - OFFSET - rct.width;
		}
		if (rct.y > intHeight) {
			rct.y = pnt.y - OFFSET;
		} else if (rct.y - fm.getHeight() < 0) {
			rct.y = fm.getHeight() + OFFSET
				- (fm.getLeading() + fm.getDescent());
		}
		g.drawString(str, rct.x, rct.y);
	}
		
	public void setColor(Color c) {
		g.setColor(c);
	}	
	
	public Color getColor() {
		return g.getColor();
	}
	
	public void setFont(Font f) {
		g.setFont(f);
	}
	
	public Font getFont() {
		return g.getFont();
	}
	
	public FontMetrics getFontMetrics() {
		return g.getFontMetrics();
	}
	
	public double getX1() { return dblX1; }
	public double getY1() { return dblY1; }
	public double getX2() { return dblX2; }
	public double getY2() { return dblY2; }
	
	// Rechnet die virtuellen in reale Koordianten um
	private Point virtual2real(double x, double y) {
		return new Point(
			(new java.lang.Long(Math.round((x - dblX1) * dblXFactor))).intValue(),
			(new java.lang.Long(Math.round((dblY2 - y) * dblYFactor))).intValue()
		);
	}
}
