/*
Copyright (C) 2003 Bj\"orn Hoffmeister

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package de.uni_luebeck.tcs.visualization;

import java.awt.*;

public interface AxesGraphicsInterface {
	// Parameter:
	// - g: Graphicsobjekt, in welches gezeichnet wird
	// - background: Farbe des Hintergrundes
	// - dblX1, dblY1, dblX2, dblY2: 
	//	kleinester bzw. groesster X- und Y-Wert in der angezeigten Achse
	// - width, height: Weite und Breite in Pixel, die zum Zeichnen zur Verfuegung stehen
	/*public AxesGraphicsInterface(
		Graphics g, 
		Color clrBackground, Color clrGrid,
		Font f,
		double dblX1, double dblY1, double dblX2, double dblY2, 
		int width, int height
	);
	*/
	
	// Loescht das Bild und fuellt es mit der Farbe background
	public void clear();
	
	// Zeichnet ein Gitternetz fuer die uebergebenen x- und y-Werte
	public void drawGrid(double[] x, double[] y);
	
	// zeichnet einen Punkt
	public void drawPoint(double x, double y);
	
	// zeichnet einen Punkt, wobei die Form des Punktes durch den
	// uebergebenen Character c bestimmt wird
	// Unterstuetzt werden sollten zumindest ./*/+/o,
	// wobei . einem ein Pixel grossen Punkt entspricht
	public void drawPoint(double x, double y, char c);
	
	// Zeichnet eine Linie zwischen den beiden Punkten
	public void drawLine(double x1, double y1, double x2, double y2);
	
	// Zeichnet eine Kurve mit linearer Interpolation zwischen den uebergebenen
	// Punkten
	public void drawLine(double[] x, double[] y);
	
	// Zeichnet und fuellt einen Kreis, wobei d den Durchmesser in Pixeln angibt
	public void fillCircle(double dblX, double dblY, int d);
	
	// Zeichnet eine Kurve mit linearer Interpolation zwischen den uebergebenen
	// Punkten und faerbt den Bereich, den die Kurve einschliesst, wenn man Start-
	// und Endpunkt mit einer Linie verbindet
	public void fillLine(double[] dblX, double[] dblY);

    // Zeichnet und fuellt ein Rechteck
	public void fillRect(double dblX, double dblY, double dblWidth, double dblHeight);

	// Zeichent den String neben dem Punkt
	public void drawString(double x, double y, String str);
	
	// setzt die Vordergrundfarbe
	public void setColor(Color c);
	
	// gibt die Vordergrundfarbe zurueck
	public Color getColor();
	
	// setzt den Font fuer Schriftausgaben
	public void setFont(Font f);
	
	// gibt den gestzten Font zurueck
	public Font getFont();

	// gibt ein FontMetrics-Objekt passend zum gesetzten Font zurueck
	public FontMetrics getFontMetrics();

	// Liefert die Koordianten links unten und rechts oben, des Koordinatensystems, in das gezeichnet wird
	public double getX1();
	public double getY1();
	public double getX2();
	public double getY2();
}
