/*
Copyright (C) 2003 Bj\"orn Hoffmeister

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package de.uni_luebeck.tcs.visualization;

import java.awt.Color;

// Stellt die Differenz von zwei Kurven da, die an denselben Punkten X berechnet wurden
// Die beiden Kurven werden durch ein gemeinsames Array der x-Werte und zwei Arrays fuer
// die y-Werte repraesentiert
public class FilledDifferenceGraph extends AbstractLine {
	protected double[] dblX, dblY1, dblY2, dblXDiff, dblYDiff;
	protected Color clrGraph2 = null;
	protected Color clrFill = null;

    public FilledDifferenceGraph(double[] dblX, double[] dblY1, double[] dblY2) throws Exception {
		if (dblX == null)
			throw new Exception("Arrays must not be null");
		if (dblX.length == 0)
			throw new Exception("Arrays must have at least one element");
		if ((dblX.length != dblY1.length) || (dblX.length != dblY2.length))
			throw new Exception("Arrays must have the same length");
		this.dblX = dblX;
		this.dblY1 = dblY1;
		this.dblY2 = dblY2;
		int intLength = 2*dblX.length;
		this.dblXDiff = new double[intLength];
		this.dblYDiff = new double[intLength];
		System.arraycopy(dblX, 0, dblXDiff, 0, dblX.length);
		for (int i=0; i < dblX.length; i++) {
		    dblXDiff[intLength - i - 1] = dblX[i];
		    if (dblY1[i] > dblY2[i]) {
		        dblYDiff[i] = dblY1[i];
		        dblYDiff[intLength - i - 1] = dblY2[i];
		    } else {
		        dblYDiff[i] = dblY2[i];
		        dblYDiff[intLength - i - 1] = dblY1[i];
		    }
		}
		this.dblXMax = getMax(dblX);
		this.dblXMin = getMin(dblX);
		this.dblYMax = Math.max(getMax(dblY1), getMax(dblY2));
		this.dblYMin = Math.min(getMin(dblY1), getMin(dblY2));
	}
		
	public void drawLine(AxesGraphicsInterface agr) {
		Color clrFill = this.clrFill;
		Color clrGraph1 = this.getColor();
		Color clrGraph2 = this.clrGraph2;
		if (clrFill == null) {
			clrFill = new Color(
				agr.getColor().getRed(), 
				agr.getColor().getGreen(), 
				agr.getColor().getBlue(), 
				agr.getColor().getAlpha() / 8
			);
		}
		if (clrGraph2 == null) {
		    clrGraph2 = new Color(~clrGraph1.getRGB());
		}		
		agr.setColor(clrFill);
		agr.fillLine(dblXDiff, dblYDiff);
		agr.setColor(clrGraph2);
		agr.drawLine(dblX, dblY2);
		agr.setColor(clrGraph1);
		agr.drawLine(dblX, dblY1);
	}
	
	// setzt die Farbe des zweiten Graphen
	public void setColor2(Color clrNew) { this.clrGraph2 = clrNew; }
	
	public Color getColor2() { return this.clrGraph2; }

	// setzt die Farbe zum fuellen der Differenz
	public void setFillColor(Color clrNew) { this.clrFill = clrNew; }
	
	public Color getFillColor() { return this.clrFill; }

}
