/*
Copyright (C) 2003 Bj\"orn Hoffmeister

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package de.uni_luebeck.tcs.visualization;

import java.awt.Color;
import java.util.Vector;

// Stellt eine Kurve da, die mit einer Farbe gefuellt wird
// Repraesentiert wird sie durch zwei Arrays mit den x- und y-Werten der einzelnen
// Punkte.
public class FilledGraph extends Graph {
	protected Color clrFill = null;
	protected double[] dblXFilled;
	protected double[] dblYFilled;
	
	public FilledGraph(double[] dblX, double[] dblY) throws Exception {
		super(dblX, dblY);
		dblXFilled = new double[dblX.length + 2];
		dblYFilled = new double[dblY.length + 2];
		System.arraycopy(dblX, 0, dblXFilled, 1, dblX.length);
		System.arraycopy(dblY, 0, dblYFilled, 1, dblY.length);
		dblXFilled[0] = dblX[0];
		dblYFilled[0] = getMinY();
		dblXFilled[dblXFilled.length-1] = dblX[dblX.length-1];
		dblYFilled[dblYFilled.length-1] = getMinY();
	}
	
	public void drawLine(AxesGraphicsInterface agr) {
		Color clrTemp = agr.getColor();
		Color clrFill = this.clrFill;
		if (clrFill == null) {
			clrFill = new Color(
				agr.getColor().getRed(), 
				agr.getColor().getGreen(), 
				agr.getColor().getBlue(), 
				agr.getColor().getAlpha() / 8
			);
		}
		agr.setColor(clrFill);
		agr.fillLine(dblXFilled, dblYFilled);
		agr.setColor(clrTemp);
		super.drawLine(agr);
	}
	
	// setzt die Farbe zum fuellen der Kurve
	public void setFillColor(Color clrNew) { this.clrFill = clrNew; }
	
	public Color getFillColor() { return this.clrFill; }

}
