/*
Copyright (C) 2003 Bj\"orn Hoffmeister

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package de.uni_luebeck.tcs.visualization;

import java.util.Vector;

// Stellt eine einfache Kurve da. 
// Repraesentiert wird sie durch zwei Arrays mit den x- und y-Werten der einzelnen
// Punkte.
public class Graph extends AbstractLine {
	public static final int LINE = 0;
	public static final int BOTH = 1;
	public static final int POINTS = 2;
	
	private int intMode = LINE;
	protected double[] dblX, dblY;
	private char chrPoint = '.';
	private Vector vctChild = new Vector(3);
	
	public Graph(double[] dblX, double[] dblY) throws Exception {
		if (dblX == null)
			throw new Exception("Arrays must not be null");
		if (dblX.length == 0)
			throw new Exception("Arrays must have at least one element");
		if (dblX.length != dblY.length)
			throw new Exception("Arrays must have the same length");
		this.dblX = dblX;
		this.dblY = dblY;
		this.dblXMax = getMax(dblX);
		this.dblXMin = getMin(dblX);
		this.dblYMax = getMax(dblY);
		this.dblYMin = getMin(dblY);
	}
	
	// Die Kurve kann in verschiedenen Modi gezeichent werden:
	// LINE: sie wird als kontinuierliche Kurve gezeichnet
	// POINTS: es werden nur die Punkte gezeichnet, wobei ein Punkt entsprechend
	//		dem in setPoint festgelegten char dargestellt wird
	// BOTH: Kombination aus LINE und POINTS
	public void setMode(int intMode) {
		this.intMode = intMode;
	}
	
	// setzt den char, durch den das Aussehen der Punkte festgelegt wird
	// (siehe setMode)
	public void setPoint(char chrPoint) {
		this.chrPoint = chrPoint;
		if ((chrPoint == '.') && (intMode == BOTH)) {
			setMode(LINE);
		} else if (intMode == LINE) {
			setMode(BOTH);
		}
	}
	
	public void drawLine(AxesGraphicsInterface agr) {
		if (intMode != POINTS)
			agr.drawLine(dblX, dblY);
		if (intMode != LINE) {
			for (int i=0; i < dblX.length; i++) {
				agr.drawPoint(dblX[i], dblY[i], chrPoint);
			}
		}
	}
}
