/*
Copyright (C) 2003 Bj\"orn Hoffmeister

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package de.uni_luebeck.tcs.visualization;

/**
* Zeichnet stueckchenweise die uebergebenen Graphen
*/

public class GraphBundle extends AbstractLine {
	private Graph[] grpBundle;
	
	public GraphBundle(double[][] dblX, double[][] dblY) throws Exception {
		if ((dblX == null) || (dblY == null))
			throw new Exception("dblX and dblY must not be null");
		if (dblX.length != dblY.length)
			throw new Exception("dblX and dblY must have the same size");
		Graph[] grpTemp = new Graph[dblX.length];
		for (int i=0; i < dblX.length; i++) {
			grpTemp[i] = new Graph(dblX[i], dblY[i]);
		}
		init(grpTemp);
	}	
	
	public GraphBundle(Graph[] grpBundle) throws Exception { init(grpBundle); }

	public void init(Graph[] grpBundle) throws Exception {
		if (grpBundle == null)
			throw new Exception("Array must not be null");
		if (grpBundle.length == 0)
			throw new Exception("Array must have at least one element");
		this.grpBundle = grpBundle;
		double[] dblMinX = new double[grpBundle.length];
		double[] dblMaxX = new double[grpBundle.length];
		double[] dblMinY = new double[grpBundle.length];
		double[] dblMaxY = new double[grpBundle.length];
		for (int i=0; i < grpBundle.length; i++) {
			dblMinX[i] = grpBundle[i].getMinX();
			dblMaxX[i] = grpBundle[i].getMaxX();
			dblMinY[i] = grpBundle[i].getMinY();
			dblMaxY[i] = grpBundle[i].getMaxY();
		}
		this.dblXMin = getMin(dblMinX);
		this.dblXMax = getMax(dblMaxX);
		this.dblYMin = getMin(dblMinY);
		this.dblYMax = getMax(dblMaxY);
	}
		

	public void setMode(int intMode) {
		for (int i=0; i < grpBundle.length; i++) {
			grpBundle[i].setMode(intMode);
		}
	}
	
	public void setPoint(char chrPoint) {
		for (int i=0; i < grpBundle.length; i++) {
			grpBundle[i].setPoint(chrPoint);
		}
	}

	// Zeichnet das Objekt
	public void drawLine(AxesGraphicsInterface agr) {
		for (int i=0; i < grpBundle.length; i++) {
			grpBundle[i].draw(agr);
		}
	}
}
