/*
Copyright (C) 2003 Bj\"orn Hoffmeister

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package de.uni_luebeck.tcs.visualization;

import java.awt.Color;

// Stellt ein Histogramm da
// Es werden die verschiedenen Hoehen der einzelnen Balken uebergeben.
// Diese koennen von einem definierten Startpunkt in der uebergebenen Breite
// uqudistant aufgereiht werden.
// Alternativ kann die Startposition und/oder Breite eines jeden Balkens mituebergeben
// werden.
public class Histogram extends AbstractLine {
	protected Color clrFill = null;
	protected double[] dblX;
	protected double[] dblWidth;
	protected double[] dblHeight;

    public Histogram(double dblBegin, double dblWidth, double[] dblHeight) throws Exception {
		if (dblHeight == null)
			throw new Exception("Height must not be null");
		this.dblHeight = dblHeight;
        this.dblX = new double[dblHeight.length];
        this.dblWidth = new double[dblHeight.length];
        for (int i=0; i < dblX.length; i++) {
            this.dblX[i] = dblBegin + i * dblWidth;
            this.dblWidth[i] = dblWidth;
        }
        init();
    }

     public Histogram(double[] dblX, double[] dblHeight) throws Exception {
		if ((dblX == null) || (dblHeight == null))
			throw new Exception("Arrays must not be null");
		if (dblX.length < 2)
			throw new Exception("x must have at least two element");
		if (dblX.length - 1 != dblHeight.length)
			throw new Exception("|x|-1 must equal |height|");
		this.dblX = new double[dblX.length - 1];
		System.arraycopy(dblX, 0, this.dblX, 0, dblX.length - 1);
		this.dblHeight = dblHeight;
        	this.dblWidth = new double[this.dblX.length];
        	for (int i=0; i < this.dblWidth.length; i++) {
            		this.dblWidth[i] = dblX[i + 1] - dblX[i];
        	}
        	init();
    }
    
     public Histogram(double[] dblX, double dblWidth, double[] dblHeight) throws Exception {
		if ((dblX == null) || (dblHeight == null))
			throw new Exception("Arrays must not be null");
		if (dblX.length == 0)
			throw new Exception("Arrays must have at least one element");
		if (dblX.length != dblHeight.length)
			throw new Exception("Arrays must have the same length");
		this.dblX = dblX;
		this.dblHeight = dblHeight;
        this.dblWidth = new double[dblHeight.length];
        for (int i=0; i < dblX.length; i++) {
            this.dblWidth[i] = dblWidth;
        }
        init();
    }
    
     public Histogram(double[] dblX, double[] dblWidth, double[] dblHeight) throws Exception {
		if ((dblX == null) || (dblHeight == null) || (dblWidth == null))
			throw new Exception("Arrays must not be null");
		if (dblX.length == 0)
			throw new Exception("Arrays must have at least one element");
		if ((dblX.length != dblWidth.length) || (dblX.length != dblHeight.length))
			throw new Exception("Arrays must have the same length");
        this.dblX = dblX;
        this.dblWidth = dblWidth;
        this.dblHeight = dblHeight;
        init();
    }
	
	protected void init() {
	    double[] dblTemp = new double[dblX.length];
	    for (int i = 0; i < dblX.length; i++) dblTemp[i] = dblX[i] + dblWidth[i];
	    this.dblXMax = getMax(dblTemp);
	    this.dblXMin = getMin(dblX);
	    this.dblYMax = getMax(dblHeight);
	    this.dblYMin = 0.0;
	}
	
    public void drawLine(AxesGraphicsInterface agr) {
		Color clrTemp = agr.getColor();
		Color clrFill = this.clrFill;
		double[] dblOutlineX = new double[4];
		double[] dblOutlineY = new double[4];
		dblOutlineY[0] = 0.0;
		dblOutlineY[3] = 0.0;
		if (clrFill == null) {
			clrFill = new Color(
				agr.getColor().getRed(), 
				agr.getColor().getGreen(), 
				agr.getColor().getBlue(), 
				agr.getColor().getAlpha() / 8
			);
		}
		for (int i = 0; i < dblX.length; i++) {		    
//		    agr.setColor(clrFill);
//		    agr.fillRect(dblX[i], 0.0, dblWidth[i], dblHeight[i]);
//		    agr.setColor(clrTemp);
		    dblOutlineX[0] = dblX[i];
		    dblOutlineX[1] = dblX[i];
		    dblOutlineX[2] = dblX[i] + dblWidth[i];
		    dblOutlineX[3] = dblX[i] + dblWidth[i];
    		dblOutlineY[1] = dblHeight[i];
	    	dblOutlineY[2] = dblHeight[i];
		    agr.setColor(clrFill);
	    	agr.fillLine(dblOutlineX, dblOutlineY);
		    agr.setColor(clrTemp);
		    agr.drawLine(dblOutlineX, dblOutlineY);
		}
	}
	
	// setzt die Farbe zum fuellen der Histogramme
	public void setFillColor(Color clrNew) { this.clrFill = clrNew; }
	
	public Color getFillColor() { return this.clrFill; }

}
