/*
Copyright (C) 2003 Bj\"orn Hoffmeister

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package de.uni_luebeck.tcs.visualization;

import java.awt.*;
import javax.swing.*;

public class JLine extends JComponent {
// constants
	// intType
	public static final int PLAIN = 0;
	public static final int ROUNDED = 1;
	public static final int RAISED_3D = 2;
	public static final int NOT_RAISED_3D = 3;	
	// intForm
	public static final int LINE_VERTICAL = 0;
	public static final int LINE_HORIZONTAL = 1;
	public static final int EDGE_NW = 2;
	public static final int EDGE_N = 3;
	public static final int EDGE_NE = 4;
	public static final int EDGE_E = 5;
	public static final int EDGE_SE = 6;
	public static final int EDGE_S = 7;
	public static final int EDGE_SW = 8;
	public static final int EDGE_W = 9;
	public static final int CROSS = 10;	
	public static final int SEPERATOR_VERTICAL = 11;
	public static final int SEPERATOR_HORIZONTAL = 12;
	
// attributes
	int intType;
	int intForm;
	int intThick;
	int intSpaceV;
	int intSpaceH;
	int intLinesVertical;
	int intLinesHorizontal;
	Color clrLine;
	
	Dimension dimMin = null;
	Dimension dimPreferred = null;

// constructor
  // public
	public JLine(int intType, int intForm) {
		this(intType, intForm, 1, 1, 1, null);
	}		

	public JLine(int intType, int intForm, int intThick, int intLinesHorizontal, int intLinesVertical, Color clrLine) {
		this.intType = intType;
		this.intForm = intForm;
		this.intThick = intThick;
		this.intLinesHorizontal = intLinesHorizontal;
		this.intLinesVertical = intLinesVertical;
		this.clrLine = clrLine;
		switch (intType) {
		case RAISED_3D:
		case NOT_RAISED_3D:
			this.intThick++;
			this.intSpaceH = 1;
			this.intSpaceV = 1;
			break;
		default:
			this.intSpaceH = 1;
			this.intSpaceV = 1;
		}
		int xThick = intLinesVertical * (intThick + intSpaceV) - intSpaceV;
		int yThick = intLinesHorizontal * (intThick + intSpaceH) - intSpaceH;
		dimMin = new Dimension(xThick, yThick);
		dimPreferred = new Dimension(xThick + 2, yThick + 2);						
	}		

// methods
  //public	
	public Dimension getMinimumSize() { return dimMin; }
	
	public Dimension getPreferredSize() { return dimPreferred; }
	
	public void paintComponent(Graphics g) {
		int w = getWidth();
		int h = getHeight();
		int offset = intThick;		
		int xThick = intLinesVertical * (intThick + intSpaceV) - intSpaceV;
		int yThick = intLinesHorizontal * (intThick + intSpaceH) - intSpaceH;
		int x = (w - xThick) / 2;
		int y = (h - yThick) / 2;
		int null_o = -offset;
		int w_o = w + (2 * offset);
		int h_o = h + (2 * offset);
		int dx = (intThick + intSpaceV);
		int dy = (intThick + intSpaceH);
		int o;
		
		// horizontal
		switch (intForm) {
		case SEPERATOR_HORIZONTAL:
			for (int i = 0; i < intLinesHorizontal; i++) {
				drawLine(g, 0, y + (i * dy), w, intThick);
			}
			break;
		case LINE_HORIZONTAL:
		case EDGE_N:
		case EDGE_S:
			for (int i = 0; i < intLinesHorizontal; i++) {
				drawLine(g, null_o, y + (i * dy), w_o, intThick);
			}
			break;
		case EDGE_E:
			for (int i = 0; i < intLinesHorizontal; i++) {
				drawLine(g, null_o, y + (i * dy), x - null_o, intThick);
			}
			break;
		case EDGE_W:
			for (int i = 0; i < intLinesHorizontal; i++) {
				drawLine(g, x + xThick, y + (i * dy), w_o - x - xThick, intThick);
			}
			break;
		case EDGE_NW:
			o = 0;
			for (int i = 0; i < intLinesHorizontal; i++) {
				drawLine(g, x + (o * dy), y + (i * dy), w_o - (x + (o * dy)), intThick);
				if (o < intLinesVertical-1) o++;
			}
			break;
		case EDGE_NE:
			o = 0;
			for (int i = 0; i < intLinesHorizontal; i++) {
				drawLine(g, null_o, y + (i * dy), x - (o * dy) - null_o + xThick, intThick);
				if (o < intLinesVertical-1) o++;
			}
			break;
		case EDGE_SW:
			o = intLinesVertical-1;
			for (int i = 0; i < intLinesHorizontal; i++) {
				drawLine(g, x + (o * dy), y + (i * dy), w_o - (x + (o * dy)), intThick);
				if (o > 0) o--;
			}
			break;
		case EDGE_SE:
			o = intLinesVertical-1;
			for (int i = 0; i < intLinesHorizontal; i++) {
				drawLine(g, null_o, y + (i * dy), x - (o * dy) - null_o + xThick, intThick);
				if (o > 0) o--;
			}
			break;
		case CROSS:
			for (int i = 0; i < intLinesHorizontal; i++) {
				drawLine(g, null_o, y + (i * dy), x - null_o, intThick);
				drawLine(g, x, y + (i * dy), w_o - x, intThick);
			}
			break;
		}

		// vertical
		switch (intForm) {
		case SEPERATOR_VERTICAL:
			for (int i = 0; i < intLinesVertical; i++) {
				drawLine(g, x + (i * dx), null_o, intThick, h_o);
			}
			break;
		case LINE_VERTICAL:
		case EDGE_E:
		case EDGE_W:
			for (int i = 0; i < intLinesVertical; i++) {
				drawLine(g, x + (i * dx), null_o, intThick, h_o);
			}
			break;
		case EDGE_N:
			for (int i = 0; i < intLinesVertical; i++) {
				drawLine(g, x + (i * dx), y + yThick, intThick, h_o - y - yThick);
			}
			break;
		case EDGE_S:
			for (int i = 0; i < intLinesVertical; i++) {
				drawLine(g, x + (i * dx), null_o, intThick, y - null_o);
			}
			break;
		case EDGE_NW:
			o = 0;
			for (int i = 0; i < intLinesVertical; i++) {
				drawLine(g, x + (i * dx), y + (o * dx), intThick, h_o - (y + (o * dx)));
				if (o < intLinesHorizontal-1) o++;
			}
			break;
		case EDGE_SW:
			o = 0;
			for (int i = 0; i < intLinesVertical; i++) {
				drawLine(g, x + (i * dx), null_o, intThick, y - (o * dx) - null_o + yThick);
				if (o < intLinesHorizontal-1) o++;
			}
			break;
		case EDGE_NE:
			o = intLinesHorizontal-1;
			for (int i = 0; i < intLinesVertical; i++) {
				drawLine(g, x + (i * dx), y + (o * dx), intThick, h_o - (y + (o * dx)));
				if (o > 0) o--;
			}
			break;
		case EDGE_SE:
			o = intLinesHorizontal-1;
			for (int i = 0; i < intLinesVertical; i++) {
				drawLine(g, x + (i * dx), null_o, intThick, y - (o * dx) - null_o + yThick);
				if (o > 0) o--;
			}
			break;
		case CROSS:
			for (int i = 0; i < intLinesVertical; i++) {
				drawLine(g, x + (i * dx), null_o, intThick, y - null_o);
				drawLine(g, x + (i * dx), y, intThick, h_o - y);
			}
			break;
		}
	}	

  // protected	
	protected void drawLine(Graphics g, int x, int y, int w, int h) {
// System.out.println("x = " + x + ", y = " + y);
		if (clrLine != null) { 
			g.setColor(clrLine); 
		} else {
			switch (intType) {
			case PLAIN:
			case ROUNDED:
				g.setColor(getForeground());
				break;
			case RAISED_3D:
			case NOT_RAISED_3D:
				g.setColor(getBackground());
				break;
			}
		}	
		switch (intType) {
		case PLAIN:
			g.fillRect(x, y, w, h);
			break;
		case ROUNDED:		
			g.fillRoundRect(x, y, w, h, intThick / 2, intThick / 2);
			break;
		case RAISED_3D:
			g.fill3DRect(x, y, w, h, true);
			break;
		case NOT_RAISED_3D:
			g.fill3DRect(x, y, w, h, false);
			break;
		}
	}			

}
