/*
Copyright (C) 2003 Bj\"orn Hoffmeister

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package de.uni_luebeck.tcs.visualization;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.Hashtable;
import javax.swing.*;
import javax.swing.event.*;

public class JRangedInput extends JPanel 
	implements ActionListener, ChangeListener, FocusListener, PropertyChangeListener {
//constants
	//public
	public static final int DOUBLE_MODE = 0;
	public static final int INTEGER_MODE = 1;
	
	//protected
	protected static final int SLIDER_START = 0;
	protected static final int SLIDER_END = 100;
	//private

//inner classes
	public static class Constraints {
	//constants	
		public final double min;
		public final double max;
		public final double avg;
		public final double def;
		public final double lowerBound;
		public final double upperBound;
		public final double step;
		public final int mode;

	//constructors
		//public
		public Constraints(double min, double max, double def) throws Exception {
			if (min >= max) throw new Exception("min must be less than max");
			if ((def < min) || (def > max)) throw new Exception("def must be in [min, max]");
			this.min = min;
			this.max = max;
			this.def = def;
			this.avg = (max - min) / 2.0;
			this.lowerBound = min;
			this.upperBound = max;
			this.step = (max - min) / (new Integer(JRangedInput.SLIDER_END - JRangedInput.SLIDER_START)).doubleValue();
			this.mode = DOUBLE_MODE;
		}
		
		public Constraints(	double min, 
					double max, 
					double avg, 
					double def, 
					double lowerBound,
					double upperBound,
					double step) throws Exception {
			this(min, max, avg, def, lowerBound, upperBound, step, DOUBLE_MODE);
		}		

		public Constraints(	double min, 
					double max, 
					double avg, 
					double def, 
					double lowerBound,
					double upperBound,
					double step,
					int mode) throws Exception {
			if (min >= max) throw new Exception("min must be less than max");
			if ((avg <= min) || (avg >= max)) throw new Exception("avg must be in (min, max)");
			if (lowerBound >= max) throw new Exception("lowerBound must be less than max");
			if (upperBound <= min) throw new Exception("lowerBound must be greater than min");			
			if ((def < lowerBound) || (def > upperBound)) throw new Exception("def must be in [lowerBound, upperBound]");
			if (Double.isNaN(def) || Double.isInfinite(def)) throw new Exception("def must be a number");
			if (step <= 0) throw new Exception("step must be greater than 0");
			if (step > max - min)  throw new Exception("step is to big");
			if ((mode != DOUBLE_MODE) && (mode != INTEGER_MODE)) throw new Exception("unknown mode");
			this.min = min;
			this.max = max;
			this.avg = avg;
			this.def = def;
			this.lowerBound = lowerBound;
			this.upperBound = upperBound;
			this.step = step;
			this.mode = mode;
		}
	}
	
	public static class Model {
	//constants
		static final String X = "JRangedInput.X";
		static final String CONSTRAINTS = "JRangedInput.Constraints";
		//public
		//protected
		//private
	
	//attributes
		JRangedInput.Constraints jriConstraints = null;
		//public
		//protected
		//private
		private Double dblData = null;
		private PropertyChangeSupport pcsModel = new PropertyChangeSupport(this);

	//constructors
		//public
		public Model(JRangedInput.Constraints jriConstraints) {
			init(jriConstraints);
		}					

	//methods
		//public
		public void addPropertyChangeListener(PropertyChangeListener l) {
			pcsModel.addPropertyChangeListener(l);
		}		

		//private
		private void init(JRangedInput.Constraints jriConstraints) {
			this.jriConstraints = jriConstraints;
			if (dblData == null) {
				dblData = new Double(jriConstraints.def);
			} else {
				checkData();
			}				
		}					
		
		private void checkData() {
			if (jriConstraints.mode == INTEGER_MODE) {
				dblData = new Double(Math.round(dblData.doubleValue()));
			}
			if (dblData.doubleValue() < jriConstraints.lowerBound) {
				dblData = new Double(jriConstraints.lowerBound);
			} else if (dblData.doubleValue() > jriConstraints.upperBound) {
				dblData = new Double(jriConstraints.upperBound);
			}			
		}

	//getter-/setter-methods (public)
		public synchronized void setX(double x) {						
			Double dblOld = this.dblData;
			this.dblData = new Double(x);
			checkData();
			pcsModel.firePropertyChange(X, dblOld, this.dblData);
		}		
		public double getX() { return this.dblData.doubleValue(); }
		public Double getXAsObject() { return this.dblData; }

		public synchronized void setConstraints(JRangedInput.Constraints jriConstraints) {
			init(jriConstraints);
			pcsModel.firePropertyChange(CONSTRAINTS, null, this.jriConstraints);
		}
		public JRangedInput.Constraints getConstraints() { return this.jriConstraints; }
	}

//attributes
	JTextField jtfRI = null;
	JSlider jslRI = null;
	//public
	//protected
	protected Range2Range r2rTransform = null;
	//private
	private JRangedInput.Model mdlData = null;
	private int sliderMin;
	private int sliderAvg;
	private int sliderMax;
	private Component cmpNextFocus = null;
	private int numbers = 0;
	
//constructors
	//public
	public JRangedInput(String strName, JRangedInput.Constraints jriConstraints) {
		this(strName, new JRangedInput.Model(jriConstraints));
	}
	
	public JRangedInput(String strName, JRangedInput.Model mdlData) {
		this.mdlData = mdlData;
		mdlData.addPropertyChangeListener(this);
		this.numbers = (mdlData.jriConstraints.mode == DOUBLE_MODE)?2:0;

		GridBagLayout gblRI = new GridBagLayout();
		setLayout(gblRI);		
		GridBagConstraints gbcSliderLabel = new GridBagConstraints();
			gbcSliderLabel.anchor = GridBagConstraints.SOUTH;
			gbcSliderLabel.fill = GridBagConstraints.HORIZONTAL;
			gbcSliderLabel.insets = new Insets(5, 0, 0, 0);
			gbcSliderLabel.weightx = 1.0;
		GridBagConstraints gbcSliderTextField = new GridBagConstraints();
			gbcSliderTextField.anchor = GridBagConstraints.SOUTHEAST;
			gbcSliderTextField.gridwidth = GridBagConstraints.REMAINDER;
			gbcSliderTextField.insets = new Insets(5, 0, 0, 0);
			gbcSliderTextField.weightx = 0.0;
		GridBagConstraints gbcSlider = new GridBagConstraints();
			gbcSlider.anchor = GridBagConstraints.NORTH;
			gbcSlider.fill = GridBagConstraints.HORIZONTAL;
			gbcSlider.gridwidth = GridBagConstraints.REMAINDER;
			gbcSlider.insets = new Insets(0, 0, 5, 0);
			gbcSliderTextField.weightx = 1.0;
		JLabel jlbRI = new JLabel(strName, JLabel.CENTER);
		jtfRI = new JTextField("", 4);
		jslRI = new JSlider(JSlider.HORIZONTAL, SLIDER_START, SLIDER_END, SLIDER_START);
		gblRI.setConstraints(jlbRI, gbcSliderLabel);
		gblRI.setConstraints(jtfRI, gbcSliderTextField);
		gblRI.setConstraints(jslRI, gbcSlider);
		add(jlbRI);
		add(jtfRI);
		add(jslRI);
		
		init();
	}

	//protected
	//private

//methods
	//public
	//protected
	protected void init() {		
		sliderMin = SLIDER_START;
		sliderMax = (new Double(Math.round((mdlData.jriConstraints.max - mdlData.jriConstraints.min) / 
			mdlData.jriConstraints.step))).intValue() + SLIDER_START;
		sliderAvg = (sliderMax - sliderMin) / 2;
		r2rTransform = new Range2Range(sliderMin, sliderMax, mdlData.jriConstraints.min, mdlData.jriConstraints.max);
		
		jslRI.setMinimum(sliderMin);
		jslRI.setMaximum(sliderMax);
		jslRI.setMajorTickSpacing(sliderAvg);
		jslRI.setMinorTickSpacing(sliderAvg / 10);
		jslRI.setPaintTicks(true);
		Hashtable htbLabel = new Hashtable();
			switch (mdlData.jriConstraints.mode) {
			case DOUBLE_MODE:
				htbLabel.put(new Integer(sliderMin), new JLabel(Double.toString(mdlData.jriConstraints.min)));
				htbLabel.put(new Integer(sliderAvg), new JLabel(Double.toString(mdlData.jriConstraints.avg)));
				htbLabel.put(new Integer(sliderMax), new JLabel(Double.toString(mdlData.jriConstraints.max)));
				break;
			case INTEGER_MODE:
				htbLabel.put(new Integer(sliderMin), new JLabel(
					String.valueOf(new Double(mdlData.jriConstraints.min).intValue())
				));
				htbLabel.put(new Integer(sliderAvg), new JLabel(
					String.valueOf(new Double(mdlData.jriConstraints.avg).intValue())
				));
				htbLabel.put(new Integer(sliderMax), new JLabel(
					String.valueOf(new Double(mdlData.jriConstraints.max).intValue())
				));
				break;
			}			
		jslRI.setLabelTable(htbLabel);		
		jslRI.setPaintLabels(true);
		
		jtfRI.addActionListener(this);
		jtfRI.addFocusListener(this);
		jslRI.addChangeListener(this);

		setGUI(mdlData.getX());
	}	
	
	protected void setGUI(double x) {
        int intSliderPosition = r2rTransform.toIntRange(x);
        if (intSliderPosition < sliderMin) { intSliderPosition = sliderMin; }
        else if (intSliderPosition > sliderMax) { intSliderPosition = sliderMax; }
        if (jslRI.getValue() != intSliderPosition) {
        	jslRI.removeChangeListener(this);
        	jslRI.setValue(intSliderPosition);
        	jslRI.addChangeListener(this);
        }        	
		jtfRI.setText(Range2Range.double2String(x, -1, numbers));
	}
	
	//private

//getter-/setter-methods (public)
	public void setData(double x) throws Exception {
		if ((x < mdlData.jriConstraints.lowerBound) || (x > mdlData.jriConstraints.upperBound))
			throw new Exception("x must be in [" + mdlData.jriConstraints.lowerBound + ", " + 
				mdlData.jriConstraints.upperBound + "]");
		mdlData.setX(x);
	}
	public double getData() { return mdlData.getX(); }

	public void setNextFocus(Component cmpNextFocus) { this.cmpNextFocus = cmpNextFocus; }
	public Component getNextFocus() { return this.cmpNextFocus; }

	public JRangedInput.Model getModel() { return mdlData; }

//listener (public)
	// actionlistener
	public void actionPerformed(ActionEvent e) {
		mdlData.setX(Range2Range.string2double
			(jtfRI.getText(), mdlData.getX(), mdlData.jriConstraints.lowerBound,  mdlData.jriConstraints.upperBound)
		);
		jtfRI.setText(Range2Range.double2String(mdlData.getX(), -1, numbers));
		if (cmpNextFocus != null) cmpNextFocus.requestFocus();
	}
	
	// changelistener
	public void stateChanged(ChangeEvent e) {
		mdlData.setX(r2rTransform.toDoubleRange(jslRI.getValue()));
	}
	
	// focuslistener
	public void focusGained(FocusEvent e) {}
	
	public void focusLost(FocusEvent e) {
		mdlData.setX(Range2Range.string2double
			(jtfRI.getText(), mdlData.getX(), mdlData.jriConstraints.lowerBound,  mdlData.jriConstraints.upperBound)
		);
		jtfRI.setText(Range2Range.double2String(mdlData.getX(), -1, numbers));
	}
	
	// propertychangelistener
	public void propertyChange(PropertyChangeEvent e) {
		if (e.getPropertyName().equals(JRangedInput.Model.X)) {
			setGUI(((Double)e.getNewValue()).doubleValue());
		} else if (e.getPropertyName().equals(JRangedInput.Model.CONSTRAINTS)) {
			init();
		}
	}
}
