/*
Copyright (C) 2003 Bj\"orn Hoffmeister

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package de.uni_luebeck.tcs.visualization;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;

public class MFileLoader extends JPanel implements ActionListener, ListSelectionListener, MouseListener {
//constants
	//public
	public final static String UPDATE = "visualization.MFileLoader.update";
		
//inner classes
	
//attributes
	//public
	public JButton jbtUpdate = null;
	
	//protected
	//private
	// GUI
	private JList jliMFiles = null;
			
	private PropertyChangeSupport pcsMFiles = null;

	private URL urlDirectory = null;
	private String strIndexFile = null;
	private String[][] strMFiles = null;
	private boolean blnIsEmpty = true;
	private boolean blnUpdateCache = true;
	private double[][] dblCache = null;
	
	private final boolean blnIsInteractive = true;
	
//constructors
	//public
	public MFileLoader(URL urlDirectory) { 
		this(urlDirectory, "index", null); 
	}	

	public MFileLoader(URL urlDirectory, String strIndexFile, JButton jbtUpdate) {
		this.urlDirectory = urlDirectory;
		this.strIndexFile = strIndexFile;
		
		GridBagLayout gblMFileLoader = new GridBagLayout();
		setLayout(gblMFileLoader);
		setBorder(BorderFactory.createCompoundBorder(
			BorderFactory.createTitledBorder(" M-Files: "),
			BorderFactory.createEmptyBorder(5,5,5,5))
		);

		GridBagConstraints gbcList = new GridBagConstraints();
			gbcList.anchor = GridBagConstraints.WEST;
			gbcList.gridwidth = GridBagConstraints.REMAINDER;
		GridBagConstraints gbcStretch = new GridBagConstraints();
			gbcStretch.fill = GridBagConstraints.BOTH;
			gbcStretch.gridwidth = GridBagConstraints.REMAINDER;
			gbcStretch.weighty = 2.0;
		GridBagConstraints gbcButton = new GridBagConstraints();
			gbcButton.fill = GridBagConstraints.HORIZONTAL;
			gbcButton.weightx = 2.0;
			gbcButton.insets = new Insets(5, 5, 5, 5);

		jliMFiles = new JList();
		jliMFiles.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		gblMFileLoader.setConstraints(jliMFiles, gbcList);
		this.add(jliMFiles);
		jliMFiles.addMouseListener(this);
		jliMFiles.addListSelectionListener(this);
		
		if (jbtUpdate == null) {
			jbtUpdate = new JButton("Update");
			Component cmpStretch = Box.createVerticalGlue();

			gblMFileLoader.setConstraints(jbtUpdate, gbcButton);
			this.add(cmpStretch);
			this.add(jbtUpdate);
		}
		this.jbtUpdate = jbtUpdate;
		jbtUpdate.addActionListener(this);
		jbtUpdate.setEnabled(!blnIsInteractive);
				
		pcsMFiles = new PropertyChangeSupport(this);
		updateList();
		
		revalidate();
	}
	
	//protected
	//private

//methods
	//public
	public void updateList() {
		try {
			strMFiles = loadMFileList(urlDirectory, strIndexFile);			
		} catch(IOException ignore) {
			strMFiles = new String[2][0];
		}
		if (strMFiles[0].length == 0) {
			jliMFiles.setListData(new String[] { "<kein Eintrag>" });
			blnIsEmpty = true;
		} else {								
			jliMFiles.setListData(strMFiles[0]);
			blnIsEmpty = false;
		}
		jliMFiles.setSelectedIndex(0);
		blnUpdateCache = true;
	}
	
	public boolean isListEmpty() { return blnIsEmpty; }
	
	public double[][] load() throws IOException {
		int intIndex = jliMFiles.getSelectedIndex();
		if (intIndex == -1) throw new IOException("no m-file selected");
		if (blnUpdateCache) {
			dblCache = loadMFile(new URL(strMFiles[1][intIndex]));
			blnUpdateCache = false;
		}		
		return dblCache;
	}	
	
	public void addPropertyChangeListener(PropertyChangeListener pclAdd) {
		pcsMFiles.addPropertyChangeListener(pclAdd);
	}

	public void removePropertyChangeListener(PropertyChangeListener pclRemove) {
		pcsMFiles.removePropertyChangeListener(pclRemove);
	}

	// Returns a String Array A containing the listed M-Files of the file urlDirectory/strIndexFile
	// The line i in the file "index" has to look like <m-file.m> and will be transformed into
	// A[0][i] = <m-file> and A[1][i] = <urlDirectory/m-file.m>.
	public static String[][] loadMFileList(URL urlDirectory, String strIndexFile) throws IOException {
		String[][] strList = null;
		BufferedReader bfrIn = null;
		LinkedList lltBuffer = new LinkedList();
		try {
			URL urlIndex = new URL(urlDirectory.toString() + strIndexFile);
			bfrIn = new BufferedReader(new InputStreamReader(urlIndex.openStream()));
			String strLine;
			while ((strLine = bfrIn.readLine()) != null) {
				strLine = strLine.trim();
				if (strLine.toLowerCase().endsWith(".m")) lltBuffer.add(strLine);
			}			
			bfrIn.close();
		} catch(MalformedURLException mue) {
			throw new IOException("Cannot find index-file: " + mue);			
		} catch(IOException ioe) {
			try { bfrIn.close(); } catch(Exception ignore) {}
			throw new IOException("Cannot read index-file: " + ioe);
		}
		int intSize = lltBuffer.size();
		if (intSize == 0) {
			strList = new String[2][0];
		} else {
			strList = new String[2][intSize];
			ListIterator litRows = lltBuffer.listIterator(0);
			for(int i = 0; i < intSize; i++) {
				String strTmp = (String)litRows.next();
				strList[0][i] = strTmp.substring(0, strTmp.length() - 2);
				strList[1][i] = urlDirectory + strTmp;
			}
		}			
		return strList;
	}
	
	public static double[][] loadMFile(URL urlMFile) throws IOException {			
		BufferedReader bfrIn = null;
		double[][] dblM = null;
		LinkedList lltBuffer = new LinkedList();		
		try {
			bfrIn = new BufferedReader(new InputStreamReader(urlMFile.openStream()));
			String strLine = null;			
			while ((strLine = bfrIn.readLine()) != null) lltBuffer.add(strLine);
			bfrIn.close();
		} catch(IOException ioe) {
			try { bfrIn.close(); } catch(Exception ignore) {}
			throw new IOException("Could not read M-File: " + ioe);
		}		
		int intRows = lltBuffer.size();
		if (intRows == 0) {
			dblM = new double[0][0];
		} else {
			ListIterator litRows = lltBuffer.listIterator(0);		
			StringTokenizer stzRow = new StringTokenizer((String)litRows.next(), "  ");
			int intColoumns = stzRow.countTokens();
			dblM = new double[intRows][intColoumns];				
			if (intColoumns != 0) {
				for (int i = 0; i < intRows; i++) {
					for (int j = 0; j < intColoumns; j++) {
						if (stzRow.hasMoreTokens()) {
							try {
								dblM[i][j] = (new Double(stzRow.nextToken())).doubleValue();
							} catch(NumberFormatException e) {
								dblM[i][j] = Double.NaN;
							}
						} else {
							dblM[i][j] = Double.NaN;
						}
					}
					if (litRows.hasNext()) stzRow = new StringTokenizer((String)litRows.next(), "  ");
				}
			}
		}
		return dblM;
	}	

	//protected	
	protected void firePropertyChange() {
		if (!jliMFiles.isSelectionEmpty() && !blnIsEmpty)
			pcsMFiles.firePropertyChange(UPDATE, null, null);			
	}
	
	//private
	
//getter-/setter-methods (public)
	public void setDirectory(URL urlDirectory) {
		this.urlDirectory = urlDirectory;
		updateList();
	}	
	public URL getDirectory() {
		return this.urlDirectory;
	}
	
	public void setIndexFile(String strIndexFile) {
		this.strIndexFile = strIndexFile;
		updateList();
	}	
	public String getIndexFile() {
		return this.strIndexFile;
	}

	public String getSelectedFile() {
		if (!jliMFiles.isSelectionEmpty() && !blnIsEmpty) {
			return strMFiles[0][jliMFiles.getSelectedIndex()];
		} else {
			return null;
		}
	}
		
//listener (public)
	// ActionListener
	public void actionPerformed(ActionEvent e) {
		firePropertyChange();
		if (blnIsInteractive) jbtUpdate.setEnabled(false);
	}
	
	//ListSelectionListener
	public void valueChanged(ListSelectionEvent e) {
		blnUpdateCache = true;
		if (blnIsInteractive) jbtUpdate.setEnabled(true);
	}	
	
	// MouseListener
	public void mouseClicked(MouseEvent e) {
   		if (e.getClickCount() == 2) firePropertyChange();   		
	}

	public void mouseEntered(MouseEvent e) {}

	public void mouseExited(MouseEvent e) {}

	public void mousePressed(MouseEvent e) {}

	public void mouseReleased(MouseEvent e) {}
}
