/*
Copyright (C) 2003 Bj\"orn Hoffmeister

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package de.uni_luebeck.tcs.visualization;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.*;

import de.uni_luebeck.tcs.matlab.*;
import de.uni_luebeck.tcs.visualization.*;

public class PDFPanel extends JPanel implements ActionListener, PropertyChangeListener {
//constants
	//public
	public final static String UPDATE = "visualization.PDFPanel.update";
	// verschiedenen Verteilungsfunktionen
	public final static int NORM = 0;
	public final static int MIXED = 1;
	public final static int CLAW = 2;
	// Indizes fuer das boolean-array bei einem property-change
	public final static int INDEX_X_CHANGED = 0;
	public final static int INDEX_PDF_CHANGED = 1;

	//protected
	// Parameter fuer die x-Achse
	protected final static double INTERVAL_BEGIN = -3.5;
	protected final static double INTERVAL_END = 3.5;
	protected final static double INTERVAL_STEP = 0.01;

	//private
	// Parameter der Normalverteilung(en)
	private static final double MU_MIN = 0.0;
	private static final double MU_AVG = 2.0;
	private static final double MU_MAX = 4.0;
	private static final double MU_DEFAULT = 2.0;
	private static final double MU_MIN_VALUE = 0.001;
	private static final double MU_MAX_VALUE = Double.POSITIVE_INFINITY;
	private static final double MU_STEP = 0.1;

	private static final double SIGMA_MIN = 0.0;
	private static final double SIGMA_AVG = 2.0;
	private static final double SIGMA_MAX = 4.0;
	private static final double SIGMA_DEFAULT = 0.5;
	private static final double SIGMA_MIN_VALUE = 0.001;
	private static final double SIGMA_MAX_VALUE = Double.POSITIVE_INFINITY;
	private static final double SIGMA_STEP = 0.1;

	private static final double P_MIN = 0.0;
	private static final double P_AVG = 0.5;
	private static final double P_MAX = 1.0;
	private static final double P_DEFAULT = 0.5;
	private static final double P_MIN_VALUE = 0.001;
	private static final double P_MAX_VALUE = 0.999;
	private static final double P_STEP = 0.1;

	private static final double CLAW_SIGMA_MIN = 0.0;
	private static final double CLAW_SIGMA_AVG = 0.2;
	private static final double CLAW_SIGMA_MAX = 0.4;
	private static final double CLAW_SIGMA_DEFAULT = 0.1;
	private static final double CLAW_SIGMA_MIN_VALUE = 0.001;
	private static final double CLAW_SIGMA_MAX_VALUE = Double.POSITIVE_INFINITY;
	private static final double CLAW_SIGMA_STEP = 0.01;
	
//inner classes
	
//attributes
	//public
	public JButton jbtUpdate = null;
	
	//protected
	//private
	// GUI
	private JRadioButton jrbNorm = null;
	private JRadioButton jrbMixed = null;
	private JRadioButton jrbClaw = null;
	private JPanel jplMixedParam = null;
	private JPanel jplClawParam = null;
	private JRangedInput.Model mdlMu = null;
	private JRangedInput.Model mdlSigma = null;
	private JRangedInput.Model mdlP = null;
	private JRangedInput.Model mdlClawSigma = null;
	
	private PropertyChangeSupport pcsPDF = null;
	private int intPDF = NORM;
	
	private int intOldPDF = 0;
	private double dblOldMu = 0.0;
	private double dblOldSigma = 0.0;
	private double dblOldP = 0.0;
	private double dblOldClawSigma = 0.0;

	private final boolean blnIsInteractive = true;
	
//constructors
	//public
	public PDFPanel() { this(null); }	

	public PDFPanel(JButton jbtUpdate) {
		GridBagLayout gblPDFPanel = new GridBagLayout();
		setLayout(gblPDFPanel);
		setBorder(BorderFactory.createCompoundBorder(
			BorderFactory.createTitledBorder(" Verteilungsfunktion: "),
			BorderFactory.createEmptyBorder(5,5,5,5))
		);

		GridBagConstraints gbcRadioButton = new GridBagConstraints();
			gbcRadioButton.anchor = GridBagConstraints.WEST;
			gbcRadioButton.gridwidth = GridBagConstraints.REMAINDER;
		GridBagConstraints gbcPanel = new GridBagConstraints();
			gbcPanel.anchor = GridBagConstraints.WEST;
			gbcPanel.gridwidth = GridBagConstraints.REMAINDER;
			gbcPanel.insets = new Insets(0, 0, 5, 0);
		GridBagConstraints gbcPDFParam = new GridBagConstraints();
			gbcPDFParam.anchor = GridBagConstraints.WEST;
			gbcPDFParam.gridwidth = GridBagConstraints.REMAINDER;
			gbcPDFParam.insets = new Insets(0, 20, 0, 0);
		GridBagConstraints gbcStretch = new GridBagConstraints();
			gbcStretch.fill = GridBagConstraints.BOTH;
			gbcStretch.gridwidth = GridBagConstraints.REMAINDER;
			gbcStretch.weighty = 2.0;
		GridBagConstraints gbcButton = new GridBagConstraints();
			gbcButton.fill = GridBagConstraints.HORIZONTAL;
			gbcButton.weightx = 2.0;
			gbcButton.insets = new Insets(5, 5, 5, 5);

		jrbNorm = new JRadioButton("NormPDF ( N(0, 1) )");
			jrbNorm.setAlignmentX(Component.LEFT_ALIGNMENT);
			jrbNorm.setMnemonic(KeyEvent.VK_N);
			jrbNorm.setActionCommand("NORM");
		jrbNorm.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {				
				selectPDF(PDFPanel.NORM);
			}
		});	
		jrbMixed = new JRadioButton("(1 - p) * N(0, 1) + p * N(mu, sigma^2)");
			jrbMixed.setAlignmentX(Component.LEFT_ALIGNMENT);
			jrbMixed.setMnemonic(KeyEvent.VK_P);
			jrbMixed.setActionCommand("MIXED");
		jrbMixed.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				selectPDF(PDFPanel.MIXED);
			}
		});
		jrbClaw = new JRadioButton("ClawPDF");
			jrbClaw.setAlignmentX(Component.LEFT_ALIGNMENT);
			jrbClaw.setMnemonic(KeyEvent.VK_C);
			jrbClaw.setActionCommand("CLAW");
		jrbClaw.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				selectPDF(PDFPanel.CLAW);
			}
		});
		ButtonGroup bgpPDF = new ButtonGroup();
			bgpPDF.add(jrbNorm);
			bgpPDF.add(jrbMixed);
			bgpPDF.add(jrbClaw);

		JRangedInput.Constraints jrcMu = null;
		JRangedInput.Constraints jrcSigma = null;
		JRangedInput.Constraints jrcP = null;
		JRangedInput.Constraints jrcClawSigma = null;
		try {
			jrcMu = new JRangedInput.Constraints(
				MU_MIN, MU_MAX, MU_AVG, MU_DEFAULT, 
				MU_MIN_VALUE, MU_MAX_VALUE, MU_STEP
			);
			jrcSigma = new JRangedInput.Constraints(
				SIGMA_MIN, SIGMA_MAX, SIGMA_AVG, SIGMA_DEFAULT, 
				SIGMA_MIN_VALUE, SIGMA_MAX_VALUE, SIGMA_STEP
			);
			jrcP = new JRangedInput.Constraints(
				P_MIN, P_MAX, P_AVG, P_DEFAULT, 
				P_MIN_VALUE, P_MAX_VALUE, P_STEP
			);
			jrcClawSigma = new JRangedInput.Constraints(
				CLAW_SIGMA_MIN, CLAW_SIGMA_MAX, CLAW_SIGMA_AVG, CLAW_SIGMA_DEFAULT, 
				CLAW_SIGMA_MIN_VALUE, CLAW_SIGMA_MAX_VALUE, CLAW_SIGMA_STEP
			);
		} catch(Exception e) {}
		JRangedInput jriMu = new JRangedInput("mu", jrcMu);
			mdlMu = jriMu.getModel();
			if (blnIsInteractive) mdlMu.addPropertyChangeListener(this);
		JRangedInput jriSigma = new JRangedInput("sigma", jrcSigma);
			mdlSigma = jriSigma.getModel();
			if (blnIsInteractive) mdlSigma.addPropertyChangeListener(this);
		JRangedInput jriP = new JRangedInput("p", jrcP);
			mdlP = jriP.getModel();
			if (blnIsInteractive) mdlP.addPropertyChangeListener(this);
		JRangedInput jriClawSigma = new JRangedInput("sigma", jrcClawSigma);
			mdlClawSigma = jriClawSigma.getModel();
			if (blnIsInteractive) mdlClawSigma.addPropertyChangeListener(this);

		jplMixedParam = new JPanel();
			GridBagLayout gblMixedParam = new GridBagLayout();		
			jplMixedParam.setLayout(gblMixedParam);
			gblMixedParam.setConstraints(jriMu, gbcPDFParam);
			gblMixedParam.setConstraints(jriSigma, gbcPDFParam);
			gblMixedParam.setConstraints(jriP, gbcPDFParam);
			jplMixedParam.add(jriMu);
			jplMixedParam.add(jriSigma);
			jplMixedParam.add(jriP);			
		jplClawParam = new JPanel();
			GridBagLayout gblClawParam = new GridBagLayout();
			jplClawParam.setLayout(gblClawParam);
			gblClawParam.setConstraints(jriClawSigma, gbcPDFParam);
			jplClawParam.add(jriClawSigma);

		gblPDFPanel.setConstraints(jrbNorm, gbcRadioButton);
		gblPDFPanel.setConstraints(jrbMixed, gbcRadioButton);
		gblPDFPanel.setConstraints(jplMixedParam, gbcPanel);
		gblPDFPanel.setConstraints(jrbClaw, gbcRadioButton);
		gblPDFPanel.setConstraints(jplClawParam, gbcPanel);
		this.add(jrbNorm);
		this.add(jrbMixed);
		this.add(jplMixedParam);
		this.add(jrbClaw);
		this.add(jplClawParam);
		
		if (jbtUpdate == null) {
			jbtUpdate = new JButton("Update");
			Component cmpStretch = Box.createVerticalGlue();

			gblPDFPanel.setConstraints(cmpStretch, gbcStretch);
			gblPDFPanel.setConstraints(jbtUpdate, gbcButton);
			this.add(cmpStretch);
			this.add(jbtUpdate);
		}
		this.jbtUpdate = jbtUpdate;
		jbtUpdate.addActionListener(this);
		jbtUpdate.setEnabled(!blnIsInteractive);
		
		jrbNorm.setSelected(true);
		jplMixedParam.setVisible(false);
		jplClawParam.setVisible(false);
		
		pcsPDF = new PropertyChangeSupport(this);
		setOldValues();
		
		revalidate();
	}
	
	//protected
	//private

//methods
	//public
	public double[] calcX() {
		double[] dblX = null;
		switch (intPDF) {
		case NORM:
			dblX = calcNormX();
			break;
		case MIXED:
			dblX = calcMixedX();
			break;
		case CLAW:
			dblX = calcClawX();
			break;
		}
		return dblX;
	}	
	
	public double[] calcPDF(double[] dblX) {
		double[] dblPDF = null;
		switch (intPDF) {
		case NORM:
			dblPDF = calcNormPDF(dblX);
			break;
		case MIXED:
			dblPDF = calcMixedPDF(dblX);
			break;
		case CLAW:
			dblPDF = calcClawPDF(dblX);
			break;
		}
		return dblPDF;
	}	
	
	public double[] calcCDF(double[] dblX) {
		double[] dblCDF = null;
		switch (intPDF) {
		case NORM:
			dblCDF = calcNormCDF(dblX);
			break;
		case MIXED:
			dblCDF = calcMixedCDF(dblX);
			break;
		case CLAW:
			dblCDF = calcClawCDF(dblX);
			break;
		}
		return dblCDF;
	}	
	
	public double[] calcRandomSample(int n) {
		double[] dblSample = null;
		switch (intPDF) {
		case NORM:
			dblSample = calcNormRandomSample(n);
			break;
		case MIXED:
			dblSample = calcMixedRandomSample(n);
			break;
		case CLAW:
			dblSample = calcClawRandomSample(n);
			break;
		}
		return dblSample;
	}
	
	public double calcQuantile(double gamma) {
		double dblGammaQuantile = 0.0;
		switch (intPDF) {
		case NORM:
			dblGammaQuantile = NormalDist.norminv(gamma);
			break;
		case MIXED:
			dblGammaQuantile = calcMixedQuantile(gamma);
			break;
		case CLAW:
			dblGammaQuantile = calcClawQuantile(gamma);
			break;
		}
		return dblGammaQuantile;
	}
		
	
	public void addPropertyChangeListener(PropertyChangeListener pclAdd) {
		pcsPDF.addPropertyChangeListener(pclAdd);
	}

	public void removePropertyChangeListener(PropertyChangeListener pclRemove) {
		pcsPDF.removePropertyChangeListener(pclRemove);
	}
	
	//protected	
	protected void firePropertyChange() {
		boolean blnIsChanged = false;
		boolean[] blnChanged = new boolean[] {false, false};
		if (intPDF != intOldPDF) {
			blnChanged[INDEX_X_CHANGED] = true;
			blnChanged[INDEX_PDF_CHANGED] = true;
			blnIsChanged = true;
		} else if (
			(mdlMu.getX() != dblOldMu) || 
			(mdlSigma.getX() != dblOldSigma) || 
			(mdlP.getX() != dblOldP)
		) {
			blnChanged[INDEX_X_CHANGED] = true;
			blnChanged[INDEX_PDF_CHANGED] = true;
			blnIsChanged = true;
		} else if (mdlClawSigma.getX() != dblOldClawSigma) {
			blnChanged[INDEX_PDF_CHANGED] = true;
			if (
				((mdlClawSigma.getX() <= 0.1) && (dblOldClawSigma > 0.1)) ||
				((mdlClawSigma.getX() > 0.1) && (dblOldClawSigma <= 0.1))
			) {
				blnChanged[INDEX_X_CHANGED] = true;
			}			
			blnIsChanged = true;
		}
		if (blnIsInteractive) {
			pcsPDF.firePropertyChange(UPDATE, null, blnChanged);
			jbtUpdate.setEnabled(false);
		} else {			
			if (blnIsChanged) pcsPDF.firePropertyChange(UPDATE, null, blnChanged);
		}		
		setOldValues();
	}
	
	protected void selectPDF(int intPDF) {
		this.intPDF = intPDF;
		switch (intPDF) {		
		case PDFPanel.NORM:
			jrbNorm.setSelected(true);
			jplMixedParam.setVisible(false);
			jplClawParam.setVisible(false);
			break;
		case PDFPanel.MIXED:
			jrbMixed.setSelected(true);
			jplMixedParam.setVisible(true);
			jplClawParam.setVisible(false);
			break;
		case PDFPanel.CLAW:
			jrbClaw.setSelected(true);
			jplMixedParam.setVisible(false);
			jplClawParam.setVisible(true);
			break;
		}
		if (blnIsInteractive) jbtUpdate.setEnabled(true);		
		revalidate();
	}	
	
	//private
	private void setOldValues() {
		intOldPDF = intPDF;
		dblOldMu = mdlMu.getX();
		dblOldSigma = mdlSigma.getX();
		dblOldP = mdlP.getX();
		dblOldClawSigma = mdlClawSigma.getX();
	}	
	
	// Berechnung der X-Werte
	private double[] calcNormX() {
		int n = (new Double(Math.floor((INTERVAL_END - INTERVAL_BEGIN) / INTERVAL_STEP))).intValue() + 1;
		double[] dblX = new double[n];
		int i = 0;
		for (double d = INTERVAL_BEGIN; d <= INTERVAL_END; d += INTERVAL_STEP) {
			dblX[i++] = d;
		}
		return dblX;
	}	
	
	private double[] calcMixedX() {
		int n = (new Double(Math.floor((INTERVAL_END - INTERVAL_BEGIN) / INTERVAL_STEP))).intValue() + 1;
		double[] dblX = new double[2*n];
		int i = 0;
		double dblMu = mdlMu.getX();
		double dblSigma = mdlSigma.getX();
		for (double d = INTERVAL_BEGIN; d <= INTERVAL_END; d += INTERVAL_STEP) {
			dblX[i++] = d;
			dblX[i++] = dblMu + dblSigma * d;			
		}
		Sort.sort(dblX);
		return dblX;
	}	
	
	private double[] calcClawX() {
		double dblSigma = mdlClawSigma.getX();
		if (dblSigma > 0.1) return calcNormX();

		int n = (new Double(Math.floor((INTERVAL_END - INTERVAL_BEGIN) / INTERVAL_STEP))).intValue() + 1;
		int m = n / 5;
		double[] dblX = new double[n + 5 * m];
		int i = 0;
		for (double d = INTERVAL_BEGIN; d <= INTERVAL_END; d += INTERVAL_STEP) {
			dblX[i++] = d;
		}
		double dblStart = INTERVAL_BEGIN * dblSigma - 1.0;
		double dblEnd = INTERVAL_END * dblSigma - 1.0;
		double dblStep = (dblEnd - dblStart) / (new Integer(m - 1)).doubleValue();
		for (int j = 0; j < 5; j++) {
			for (double d = dblStart; d <= dblEnd; d += dblStep) {
				dblX[i++] = d;
			}
			dblStart += 0.5;
			dblEnd += 0.5;
		}
		Sort.sort(dblX);
		return dblX;
	}

	// Berechnung der PDF-Werte
	private double[] calcNormPDF(double[] dblX) {
		double[] dblPDF = new double[dblX.length];
		for (int i = 0; i < dblX.length; i++) {
			dblPDF[i] = NormalDist.normPDF(dblX[i]);
		}
		return dblPDF;
	}

	private double[] calcMixedPDF(double[] dblX) {
		double[] dblPDF = new double[dblX.length];
		double dblMu = mdlMu.getX();
		double dblSigma = mdlSigma.getX();
		double dblP = mdlP.getX();
		for (int i = 0; i < dblX.length; i++) {
			dblPDF[i] = 
				(1 - dblP) * NormalDist.normPDF(dblX[i]) + 
				dblP * NormalDist.normPDF((dblX[i] - dblMu) / dblSigma) / dblSigma;
		}
		return dblPDF;
	}

	private double[] calcClawPDF(double[] dblX) {
		double[] dblPDF = new double[dblX.length];
		double dblSigma = mdlClawSigma.getX();
		for (int i = 0; i < dblX.length; i++) {
			dblPDF[i] = 
				0.5 * NormalDist.normPDF(dblX[i]) +
				NormalDist.normPDF((dblX[i] - 1.0) / dblSigma) * 0.1 / dblSigma +
				NormalDist.normPDF((dblX[i] - 0.5) / dblSigma) * 0.1 / dblSigma +
				NormalDist.normPDF(dblX[i] / dblSigma) * 0.1 / dblSigma +
				NormalDist.normPDF((dblX[i] + 0.5) / dblSigma) * 0.1 / dblSigma +
				NormalDist.normPDF((dblX[i] + 1.0) / dblSigma) * 0.1 / dblSigma;
		}
		return dblPDF;
	}
	
	// Berechnung der CDF-Werte
	private double[] calcNormCDF(double[] dblX) {
		double[] dblCDF = new double[dblX.length];
		for (int i = 0; i < dblX.length; i++) {
			dblCDF[i] = NormalDist.normCDF(dblX[i]);
		}
		return dblCDF;
	}

	private double[] calcMixedCDF(double[] dblX) {
		double[] dblCDF = new double[dblX.length];
		double dblMu = mdlMu.getX();
		double dblSigma = mdlSigma.getX();
		double dblP = mdlP.getX();
		for (int i = 0; i < dblX.length; i++) {
			dblCDF[i] = 
				(1 - dblP) * NormalDist.normCDF(dblX[i]) + 
				dblP * NormalDist.normCDF((dblX[i] - dblMu) / dblSigma);
		}
		return dblCDF;
	}

	private double[] calcClawCDF(double[] dblX) {
		double[] dblCDF = new double[dblX.length];
		double dblSigma = mdlClawSigma.getX();
		for (int i = 0; i < dblX.length; i++) {
			dblCDF[i] = 
				0.5 * NormalDist.normCDF(dblX[i]) +
				NormalDist.normCDF((dblX[i] - 1.0) / dblSigma) * 0.1 +
				NormalDist.normCDF((dblX[i] - 0.5) / dblSigma) * 0.1 +
				NormalDist.normCDF(dblX[i] / dblSigma) * 0.1 +
				NormalDist.normCDF((dblX[i] + 0.5) / dblSigma) * 0.1 +
				NormalDist.normCDF((dblX[i] + 1.0) / dblSigma) * 0.1;
		}
		return dblCDF;
	}

	// Berechnung eines zufaelligen Samples der groesse n
	private double[] calcNormRandomSample(int n) {
		double[] dblSample = new double[n];
		for (int i = 0; i < n; i++) {
			dblSample[i] = NormalDist.norminv(Math.random());
		}
		Sort.sort(dblSample);
		return dblSample;
	}		
	
	private double[] calcMixedRandomSample(int n) {
		double[] dblSample = new double[n];		
		double dblMu = mdlMu.getX();
		double dblSigma = mdlSigma.getX();
		double dblP = mdlP.getX();
		for (int i = 0; i < n; i++) {
			double dblY = Math.random();
			if (dblY <= 1 - dblP) {
				dblSample[i] = NormalDist.norminv(dblY / (1 - dblP));
			} else {
				dblSample[i] = dblMu + dblSigma * NormalDist.norminv((dblY -1 + dblP) / dblP);
			}
		}
		Sort.sort(dblSample);
		return dblSample;
	}
	
	private double[] calcClawRandomSample(int n) {
		double[] dblSample = new double[n];
		double dblSigma = mdlClawSigma.getX();
		for (int i = 0; i < n; i++) {
			double dblU = Math.random();
			if (dblU >= 0.9) {
				dblSample[i] = 1.0 + dblSigma * Math.random();
			} else if (dblU >= 0.8) {
				dblSample[i] = 0.5 + dblSigma * Math.random();
			} else if (dblU >= 0.7) {
				dblSample[i] = dblSigma * Math.random();
			} else if (dblU >= 0.6) {
				dblSample[i] = -0.5 + dblSigma * Math.random();
			} else if (dblU >= 0.5) {
				dblSample[i] = -1.0 + dblSigma * Math.random();
			} else {
				dblSample[i] = NormalDist.norminv(Math.random());
			}				
		}
		Sort.sort(dblSample);
		return dblSample;
	}
	
	private double calcMixedQuantile(double dblGamma) {
		// exactness
		double dblEps = 0.0001;
		
		// look for a suitable interval
		double[] dblA = new double[1];
		double[] dblB = new double[1];
		double[] dblPa = new double[1];
		double[] dblPb = new double[1];
		dblA[0] = -1.0;
		dblB[0] = 1.0;
		dblPa = calcMixedCDF(dblA);
		dblPb = calcMixedCDF(dblB);		
		while (dblPa[0] > dblGamma) {
			double dblDelta = dblB[0] - dblA[0];
			dblB[0] = dblA[0];
			dblPb[0] = dblPa[0];
			dblA[0] = dblA[0] - dblDelta;
			dblPa = calcMixedCDF(dblA);
		}
		while (dblPb[0] < dblGamma) {
			double dblDelta = dblB[0] - dblA[0];
			dblA[0] = dblB[0];
			dblPa[0] = dblPb[0];
			dblB[0] = dblB[0] + dblDelta;
			dblPb = calcMixedCDF(dblB);
		}
		
		// find quantile using bisection
		while (((dblB[0] - dblA[0]) >= dblEps) && ((dblPb[0] - dblPa[0]) >= dblEps)) {
			double[] dblM = new double[1];
			dblM[0] = (dblA[0] + dblB[0]) / 2;
			double[] dblPm = calcMixedCDF(dblM);
			if (dblPm[0] < dblGamma) {
				dblA[0] = dblM[0];
				dblPa[0] = dblPm[0];
			} else {
				dblB[0] = dblM[0];
				dblPb[0] = dblPm[0];
			}
		}
		
		return dblA[0];
	}
	
	private double calcClawQuantile(double dblGamma) {
		// exactness
		double dblEps = 0.0001;
		
		// look for a suitable interval
		double[] dblA = new double[1];
		double[] dblB = new double[1];
		double[] dblPa = new double[1];
		double[] dblPb = new double[1];
		dblA[0] = -1.0;
		dblB[0] = 1.0;
		dblPa = calcClawCDF(dblA);
		dblPb = calcClawCDF(dblB);		
		while (dblPa[0] > dblGamma) {
			double dblDelta = dblB[0] - dblA[0];
			dblB[0] = dblA[0];
			dblPb[0] = dblPa[0];
			dblA[0] = dblA[0] - dblDelta;
			dblPa = calcClawCDF(dblA);
		}
		while (dblPb[0] < dblGamma) {
			double dblDelta = dblB[0] - dblA[0];
			dblA[0] = dblB[0];
			dblPa[0] = dblPb[0];
			dblB[0] = dblB[0] + dblDelta;
			dblPb = calcClawCDF(dblB);
		}
		
		// find quantile using bisection
		while (((dblB[0] - dblA[0]) >= dblEps) && ((dblPb[0] - dblPa[0]) >= dblEps)) {
			double[] dblM = new double[1];
			dblM[0] = (dblA[0] + dblB[0]) / 2;
			double[] dblPm = calcClawCDF(dblM);
			if (dblPm[0] < dblGamma) {
				dblA[0] = dblM[0];
				dblPa[0] = dblPm[0];
			} else {
				dblB[0] = dblM[0];
				dblPb[0] = dblPm[0];
			}
		}
		
		return dblA[0];
	}

//getter-/setter-methods (public)
	public void setMu(double dblMu) {
		mdlMu.setX(dblMu);
	}	
	public double getMu() {
		return mdlMu.getX();
	}	

	public void setSigma(double dblSigma) {
		mdlSigma.setX(dblSigma);
	}			
	public double getSigma() {
		return mdlSigma.getX();
	}
	
	public void setP(double dblP) {
		mdlP.setX(dblP);
	}
	public double getP() {
		return mdlP.getX();
	}

	public void setClawSigma(double dblClawSigma) {
		mdlClawSigma.setX(dblClawSigma);
	}			
	public double getClawSigma() {
		return mdlClawSigma.getX();
	}

//listener (public)
	// ActionListener
	public void actionPerformed(ActionEvent e) {
		firePropertyChange();
	}

	// propertychangelistener
	public void propertyChange(PropertyChangeEvent e) {
		if (e.getPropertyName().equals(JRangedInput.Model.X) && blnIsInteractive) {
			jbtUpdate.setEnabled(true);
		}
	}			
}
