/*
Copyright (C) 2003 Bj\"orn Hoffmeister

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package de.uni_luebeck.tcs.visualization;

/**
* Stellt die in dblX uebergebenen Werte durch Striche der Hoehe dblHeight
* auf der X-Achse da
*/
public class Sample extends AbstractLine {
	private double[] dblX;
	private double dblY;
	private double dblHeight;
	
	// Richtet die Striche am Boden aus und zeichnet sie in einer Hoehe von intP% der Gesamthoehe
	public Sample(double[] dblX, double dblBottom, double dblTop, int intP) throws Exception {
		if (dblTop <= dblBottom)
			throw new Exception("dblTop must be larger than dblBottom");
		if (intP <= 0)
			throw new Exception("intP must be larger than zero");
		double dblHeight = (dblTop - dblBottom) * (intP / 100.0);
		init(dblX, dblBottom - dblHeight, dblHeight * 0.8);
	}	
	
	// dblX muss aufsteigend sortiert sein
	public Sample(double[] dblX, double dblY, double dblHeight) throws Exception {
		if (dblHeight < 0.0)
			throw new Exception("dblHeight must be positiv");
		init(dblX, dblY, dblHeight);
	}
	
	private void init(double[] dblX, double dblY, double dblHeight) throws Exception {
		if (dblX == null)
			throw new Exception("Arrays must not be null");
		if (dblX.length == 0)
			throw new Exception("Arrays must have at least one element");
		this.dblX = dblX;
		this.dblY = dblY;
		this.dblHeight = dblHeight;
		this.dblXMin = getMin(this.dblX);
		this.dblXMax = getMax(this.dblX);
		this.dblYMin = dblY;
		this.dblYMax = dblY + dblHeight;
	}	

	// Zeichnet das Objekt
	public void drawLine(AxesGraphicsInterface agr) {
		for (int i=0; i < this.dblX.length; i++) {
			agr.drawLine(this.dblX[i], dblY + dblHeight, this.dblX[i], dblY);
		}
	}
}
