/*
Copyright (C) 2003 Bj\"orn Hoffmeister

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package de.uni_luebeck.tcs.visualization;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.event.*;

public class VierFelderTafel extends JPanel implements ActionListener, PropertyChangeListener {
// constants
	//public
	public static final String VALUE = "VierFelderTafel.value";
	//protected
	protected static final Dimension SCREEN_SIZE = (Toolkit.getDefaultToolkit()).getScreenSize();
	//private
	private static final int VIEW_LINE_TYPE = JLine.PLAIN;
	private static final int VIEW_LINE_THICKNESS = 1;
	private static final Color VIEW_LINE_COLOR = null;
	private static final Font VIEW_BUTTON_FONT = new Font("Dialog", Font.BOLD, 9);
	private static final int EDIT_LINE_TYPE = JLine.PLAIN;
	private static final int EDIT_LINE_THICKNESS = 2;
	private static final Color EDIT_LINE_COLOR = (new JLabel()).getForeground();
	private static final Font EDIT_FONT = new Font("SansSerif", Font.BOLD, 16);

//inner classes
	public static class Model {
	//constants
		//public
		//protected
		//private
	
	//attributes
		//public
		//protected
		//private
		private int[][] intValue = new int[2][2];
		private PropertyChangeSupport pcsVierFelder = new PropertyChangeSupport(this);

	//constructors
		//public
		public Model() {
			for (int i = 0; i < 2; i++) {
				for (int j = 0; j < 2; j++) {				
					intValue[i][j] = 0;
				}
			}			
		}		
		//protected
		//private

	//methods
		//public
		public void addPropertyChangeListener(PropertyChangeListener l) {
			pcsVierFelder.addPropertyChangeListener(l);
		}		
		
		public synchronized void setValues(int[][] intValue) {
			pcsVierFelder.firePropertyChange
				(new PropertyChangeEvent(this, VierFelderTafel.VALUE, this.intValue, intValue));
			for (int i = 0; i < 2; i++) {
				for (int j = 0; j < 2; j++) {				
					this.intValue[i][j] = intValue[i][j];
				}
			}			
		}
		
		public int[][] getValues() { return intValue; }
		//protected
		//private

	//getter-/setter-methods (public)

	//listener (public)
	}
			
// attributes
	//public
	//protected
	//private
	private int[][] intVierFelder = new int[2][2];	
	private int[][] intVierFelderEdit = new int[2][2];	
	private int[] intVierFelderSummary = new int[5];	
	
	private JFrame jfrEdit = new JFrame("Vier-Felder-Tafel Editor");
	private JLabel[][] jlbVierFelder = new JLabel[2][2];
	private JLabel[] jlbViewVierFelderSummary = new JLabel[5];
	private JTextField[][] jtfVierFelder = new JTextField[2][2];
	private JLabel[] jlbVierFelderSummary = new JLabel[5];

	private Model mdlVierFelder = null;
// constructor
  // public	
	public VierFelderTafel() {
		this(new Model());
	}	
		
	public VierFelderTafel(Model mdlVierFelder) {
		this.mdlVierFelder = mdlVierFelder;
		this.mdlVierFelder.addPropertyChangeListener(this);
		
		JPanel jplEdit = new JPanel();
		setBorder(BorderFactory.createCompoundBorder(
			BorderFactory.createTitledBorder(" 4-Felder-Tafel: "),
			BorderFactory.createEmptyBorder(5, 5, 5, 5))
		);
		jplEdit.setBorder(
			BorderFactory.createEmptyBorder(8, 8, 8, 8)
		);			
		GridBagLayout gblView = new GridBagLayout();
		GridBagLayout gblEdit = new GridBagLayout();		
		setLayout(gblView);
		jplEdit.setLayout(gblEdit);
		
		GridBagConstraints gbcCorner = new GridBagConstraints();
			gbcCorner.fill = GridBagConstraints.BOTH;
			gbcCorner.weightx = 0.0;
			gbcCorner.weighty = 0.0;					
		GridBagConstraints gbcCornerEOL = new GridBagConstraints();
			gbcCornerEOL.fill = GridBagConstraints.BOTH;
			gbcCornerEOL.weightx = 0.0;
			gbcCornerEOL.weighty = 0.0;
			gbcCornerEOL.gridwidth = GridBagConstraints.REMAINDER;
		GridBagConstraints gbcHorizontal = new GridBagConstraints();
			gbcHorizontal.fill = GridBagConstraints.BOTH;
			gbcHorizontal.weightx = 1.0;
			gbcHorizontal.weighty = 0.0;					
		GridBagConstraints gbcVertical = new GridBagConstraints();
			gbcVertical.fill = GridBagConstraints.BOTH;
			gbcVertical.weightx = 0.0;
			gbcVertical.weighty = 1.0;					
		GridBagConstraints gbcVerticalEOL = new GridBagConstraints();
			gbcVerticalEOL.fill = GridBagConstraints.BOTH;
			gbcVerticalEOL.weightx = 0.0;
			gbcVerticalEOL.weighty = 1.0;					
			gbcVerticalEOL.gridwidth = GridBagConstraints.REMAINDER;
		GridBagConstraints gbcCrossOrT = new GridBagConstraints();
			gbcCrossOrT.fill = GridBagConstraints.BOTH;
			gbcCrossOrT.weightx = 0.0;
			gbcCrossOrT.weighty = 0.0;					
		GridBagConstraints gbcTEOL = new GridBagConstraints();
			gbcTEOL.fill = GridBagConstraints.BOTH;
			gbcTEOL.weightx = 0.0;
			gbcTEOL.weighty = 0.0;					
			gbcTEOL.gridwidth = GridBagConstraints.REMAINDER;		
		GridBagConstraints gbcLabel = new GridBagConstraints();
			gbcLabel.fill = GridBagConstraints.BOTH;
			gbcLabel.weightx = 1.0;
			gbcLabel.weighty = 1.0;							
		GridBagConstraints gbcTextField = new GridBagConstraints();
			gbcTextField.fill = GridBagConstraints.NONE;
			gbcTextField.weightx = 1.0;
			gbcTextField.weighty = 1.0;							
			gbcTextField.insets = new Insets(4, 4, 4, 4);
		GridBagConstraints gbcViewButton = new GridBagConstraints();
			gbcViewButton.fill = GridBagConstraints.HORIZONTAL;
			gbcViewButton.insets = new Insets(5, 0, 0, 0);
			gbcViewButton.weightx = 1.0;
			gbcViewButton.weighty = 0.0;							
			gbcViewButton.gridwidth = GridBagConstraints.REMAINDER;		
		GridBagConstraints gbcEditButton = new GridBagConstraints();
			gbcEditButton.insets = new Insets(8, 0, 0, 0);
			gbcEditButton.weightx = 1.0;
			gbcEditButton.weighty = 0.0;							
			gbcEditButton.gridwidth = GridBagConstraints.REMAINDER;		
				
		intVierFelder = mdlVierFelder.getValues();
		for (int i = 0; i < 2; i++) {
			for (int j = 0; j < 2; j++) {
				jlbVierFelder[i][j] = new JLabel("");
				jlbVierFelder[i][j].setHorizontalAlignment(JLabel.CENTER);
				jtfVierFelder[i][j] = new JTextField("", 4);
				jtfVierFelder[i][j].setHorizontalAlignment(JLabel.CENTER);
				jtfVierFelder[i][j].setMargin(new Insets(2, 2, 2, 2));
				jtfVierFelder[i][j].setFont(EDIT_FONT);
				final int fi = i; final int fj = j;
/*
				jtfVierFelder[i][j].addInputMethodListener(new InputMethodListener() {
					public void caretPositionChanged(InputMethodEvent e) {}
			
					public void inputMethodTextChanged(InputMethodEvent e) {				
						System.out.println("Changed");
						checkInput(fi, fj);
					}
				});
*/
				jtfVierFelder[i][j].addCaretListener(new CaretListener() {
					public void caretUpdate(CaretEvent e) {
						checkInput(fi, fj);
					}
				});		
			}
		}
		for (int i = 0; i < 5; i++) {
			jlbViewVierFelderSummary[i] = new JLabel("");
			jlbViewVierFelderSummary[i].setHorizontalAlignment(JLabel.CENTER);			
			jlbVierFelderSummary[i] = new JLabel("");
			jlbVierFelderSummary[i].setHorizontalAlignment(JLabel.CENTER);
			jlbVierFelderSummary[i].setFont(EDIT_FONT);
			jlbVierFelderSummary[i].setMinimumSize(jtfVierFelder[0][0].getMinimumSize());
			jlbVierFelderSummary[i].setPreferredSize(jtfVierFelder[0][0].getPreferredSize());
			jlbVierFelderSummary[i].setMaximumSize(jtfVierFelder[0][0].getMaximumSize());
//			jlbVierFelderSummary[i].setSize(jtfVierFelder[0][0].getSize());
		}
		
		setViewValues(mdlVierFelder.getValues());
		
		JLine jlnTmp;
		// View
		// 1.row
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.EDGE_NW, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcCorner);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_HORIZONTAL, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcHorizontal);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.EDGE_N, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcCrossOrT);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_HORIZONTAL, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcHorizontal);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.EDGE_N, VIEW_LINE_THICKNESS, 1, 2, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcCrossOrT);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_HORIZONTAL, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcHorizontal);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.EDGE_NE, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcCornerEOL);
		add(jlnTmp);
		// 2.row
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_VERTICAL, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcVertical);
		add(jlnTmp);
		gblView.setConstraints(jlbVierFelder[0][0], gbcLabel);
		add(jlbVierFelder[0][0]);		
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_VERTICAL, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcVertical);
		add(jlnTmp);
		gblView.setConstraints(jlbVierFelder[0][1], gbcLabel);
		add(jlbVierFelder[0][1]);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_VERTICAL, VIEW_LINE_THICKNESS, 1, 2, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcVertical);
		add(jlnTmp);
		gblView.setConstraints(jlbViewVierFelderSummary[0], gbcLabel);
		add(jlbViewVierFelderSummary[0]);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_VERTICAL, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcVerticalEOL);
		add(jlnTmp);
		// 3.row
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.EDGE_W, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcCrossOrT);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_HORIZONTAL, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcHorizontal);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.CROSS, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcCrossOrT);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_HORIZONTAL, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcHorizontal);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.CROSS, VIEW_LINE_THICKNESS, 1, 2, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcCrossOrT);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_HORIZONTAL, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcHorizontal);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.EDGE_E, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcTEOL);
		add(jlnTmp);
		// 4.row
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_VERTICAL, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcVertical);
		add(jlnTmp);
		gblView.setConstraints(jlbVierFelder[1][0], gbcLabel);
		add(jlbVierFelder[1][0]);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_VERTICAL, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcVertical);
		add(jlnTmp);
		gblView.setConstraints(jlbVierFelder[1][1], gbcLabel);
		add(jlbVierFelder[1][1]);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_VERTICAL, VIEW_LINE_THICKNESS, 1, 2, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcVertical);
		add(jlnTmp);
		gblView.setConstraints(jlbViewVierFelderSummary[1], gbcLabel);
		add(jlbViewVierFelderSummary[1]);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_VERTICAL, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcVerticalEOL);
		add(jlnTmp);
		// 5.row
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.EDGE_W, VIEW_LINE_THICKNESS, 2, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcCorner);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_HORIZONTAL, VIEW_LINE_THICKNESS, 2, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcHorizontal);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.CROSS, VIEW_LINE_THICKNESS, 2, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcCrossOrT);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_HORIZONTAL, VIEW_LINE_THICKNESS, 2, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcHorizontal);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.CROSS, VIEW_LINE_THICKNESS, 2, 2, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcCrossOrT);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_HORIZONTAL, VIEW_LINE_THICKNESS, 2, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcHorizontal);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.EDGE_E, VIEW_LINE_THICKNESS, 2, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcCornerEOL);
		add(jlnTmp);
		// 6.row
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_VERTICAL, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcVertical);
		add(jlnTmp);
		gblView.setConstraints(jlbViewVierFelderSummary[2], gbcLabel);
		add(jlbViewVierFelderSummary[2]);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_VERTICAL, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcVertical);
		add(jlnTmp);
		gblView.setConstraints(jlbViewVierFelderSummary[3], gbcLabel);
		add(jlbViewVierFelderSummary[3]);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_VERTICAL, VIEW_LINE_THICKNESS, 1, 2, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcVertical);
		add(jlnTmp);
		gblView.setConstraints(jlbViewVierFelderSummary[4], gbcLabel);
		add(jlbViewVierFelderSummary[4]);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_VERTICAL, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcVerticalEOL);
		add(jlnTmp);
		// 7.row
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.EDGE_SW, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcCorner);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_HORIZONTAL, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcHorizontal);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.EDGE_S, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcCrossOrT);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_HORIZONTAL, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcHorizontal);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.EDGE_S, VIEW_LINE_THICKNESS, 1, 2, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcCrossOrT);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.LINE_HORIZONTAL, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcHorizontal);
		add(jlnTmp);
		jlnTmp = new JLine(VIEW_LINE_TYPE, JLine.EDGE_SE, VIEW_LINE_THICKNESS, 1, 1, VIEW_LINE_COLOR);
		gblView.setConstraints(jlnTmp, gbcCornerEOL);
		add(jlnTmp);
		// 8.row
		JButton jbtEdit = new JButton("Edit");
		jbtEdit.setForeground((new JLabel()).getForeground());
		// jbtEdit.setFont(VIEW_BUTTON_FONT);
		gblView.setConstraints(jbtEdit, gbcViewButton);
		add(jbtEdit);		
		jbtEdit.addActionListener(this);
		
		// Edit
		// 1.row
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.EDGE_NW, EDIT_LINE_THICKNESS, 2, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcCorner);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_HORIZONTAL, EDIT_LINE_THICKNESS, 2, 1, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcHorizontal);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.EDGE_N, EDIT_LINE_THICKNESS, 2, 1, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcCrossOrT);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_HORIZONTAL, EDIT_LINE_THICKNESS, 2, 1, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcHorizontal);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.EDGE_N, EDIT_LINE_THICKNESS, 2, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcCrossOrT);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_HORIZONTAL, EDIT_LINE_THICKNESS, 2, 1, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcHorizontal);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.EDGE_NE, EDIT_LINE_THICKNESS, 2, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcCornerEOL);
		jplEdit.add(jlnTmp);
		// 2.row
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_VERTICAL, EDIT_LINE_THICKNESS, 1, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcVertical);
		jplEdit.add(jlnTmp);
		gblEdit.setConstraints(jtfVierFelder[0][0], gbcTextField);
		jplEdit.add(jtfVierFelder[0][0]);		
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_VERTICAL, EDIT_LINE_THICKNESS, 1, 1, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcVertical);
		jplEdit.add(jlnTmp);
		gblEdit.setConstraints(jtfVierFelder[0][1], gbcTextField);
		jplEdit.add(jtfVierFelder[0][1]);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_VERTICAL, EDIT_LINE_THICKNESS, 1, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcVertical);
		jplEdit.add(jlnTmp);
		gblEdit.setConstraints(jlbVierFelderSummary[0], gbcTextField);
		jplEdit.add(jlbVierFelderSummary[0]);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_VERTICAL, EDIT_LINE_THICKNESS, 1, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcVerticalEOL);
		jplEdit.add(jlnTmp);
		// 3.row
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.EDGE_W, EDIT_LINE_THICKNESS, 1, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcCrossOrT);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_HORIZONTAL, EDIT_LINE_THICKNESS, 1, 1, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcHorizontal);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.CROSS, EDIT_LINE_THICKNESS, 1, 1, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcCrossOrT);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_HORIZONTAL, EDIT_LINE_THICKNESS, 1, 1, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcHorizontal);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.CROSS, EDIT_LINE_THICKNESS, 1, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcCrossOrT);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_HORIZONTAL, EDIT_LINE_THICKNESS, 1, 1, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcHorizontal);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.EDGE_E, EDIT_LINE_THICKNESS, 1, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcTEOL);
		jplEdit.add(jlnTmp);
		// 4.row
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_VERTICAL, EDIT_LINE_THICKNESS, 1, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcVertical);
		jplEdit.add(jlnTmp);
		gblEdit.setConstraints(jtfVierFelder[1][0], gbcTextField);
		jplEdit.add(jtfVierFelder[1][0]);		
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_VERTICAL, EDIT_LINE_THICKNESS, 1, 1, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcVertical);
		jplEdit.add(jlnTmp);
		gblEdit.setConstraints(jtfVierFelder[1][1], gbcTextField);
		jplEdit.add(jtfVierFelder[1][1]);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_VERTICAL, EDIT_LINE_THICKNESS, 1, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcVertical);
		jplEdit.add(jlnTmp);
		gblEdit.setConstraints(jlbVierFelderSummary[1], gbcTextField);
		jplEdit.add(jlbVierFelderSummary[1]);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_VERTICAL, EDIT_LINE_THICKNESS, 1, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcVerticalEOL);
		jplEdit.add(jlnTmp);
		// 5.row
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.EDGE_W, EDIT_LINE_THICKNESS, 2, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcCrossOrT);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_HORIZONTAL, EDIT_LINE_THICKNESS, 2, 1, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcHorizontal);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.CROSS, EDIT_LINE_THICKNESS, 2, 1, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcCrossOrT);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_HORIZONTAL, EDIT_LINE_THICKNESS, 2, 1, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcHorizontal);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.CROSS, EDIT_LINE_THICKNESS, 2, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcCrossOrT);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_HORIZONTAL, EDIT_LINE_THICKNESS, 2, 1, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcHorizontal);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.EDGE_E, EDIT_LINE_THICKNESS, 2, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcTEOL);
		jplEdit.add(jlnTmp);
		// 6.row
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_VERTICAL, EDIT_LINE_THICKNESS, 1, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcVertical);
		jplEdit.add(jlnTmp);
		gblEdit.setConstraints(jlbVierFelderSummary[2], gbcTextField);
		jplEdit.add(jlbVierFelderSummary[2]);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_VERTICAL, EDIT_LINE_THICKNESS, 1, 1, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcVertical);
		jplEdit.add(jlnTmp);
		gblEdit.setConstraints(jlbVierFelderSummary[3], gbcTextField);
		jplEdit.add(jlbVierFelderSummary[3]);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_VERTICAL, EDIT_LINE_THICKNESS, 1, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcVertical);
		jplEdit.add(jlnTmp);
		gblEdit.setConstraints(jlbVierFelderSummary[4], gbcTextField);
		jplEdit.add(jlbVierFelderSummary[4]);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_VERTICAL, EDIT_LINE_THICKNESS, 1, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcVerticalEOL);
		jplEdit.add(jlnTmp);
		// 7.row
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.EDGE_SW, EDIT_LINE_THICKNESS, 2, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcCorner);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_HORIZONTAL, EDIT_LINE_THICKNESS, 2, 1, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcHorizontal);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.EDGE_S, EDIT_LINE_THICKNESS, 2, 1, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcCrossOrT);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_HORIZONTAL, EDIT_LINE_THICKNESS, 2, 1, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcHorizontal);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.EDGE_S, EDIT_LINE_THICKNESS, 2, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcCrossOrT);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.LINE_HORIZONTAL, EDIT_LINE_THICKNESS, 2, 1, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcHorizontal);
		jplEdit.add(jlnTmp);
		jlnTmp = new JLine(EDIT_LINE_TYPE, JLine.EDGE_SE, EDIT_LINE_THICKNESS, 2, 2, EDIT_LINE_COLOR);
		gblEdit.setConstraints(jlnTmp, gbcCornerEOL);
		jplEdit.add(jlnTmp);
		// 8.row
		JPanel jplButtons = new JPanel(new GridLayout(1, 2, 2, 0));
		JButton jbtOK = new JButton("OK");
		JButton jbtCancel = new JButton("     Cancel     ");
		jplButtons.add(jbtOK);
		jplButtons.add(jbtCancel);
		gblEdit.setConstraints(jplButtons, gbcEditButton);
		jplEdit.add(jplButtons);
		jbtOK.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				showEdit(false);
				getEditValues();
			}
		});
		jbtOK.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				if (e.getKeyCode() == KeyEvent.VK_ENTER) {
					showEdit(false);
					getEditValues();
				}
			}
		});					
		jbtCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				showEdit(false);
			}
		});
		jbtCancel.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				if (e.getKeyCode() == KeyEvent.VK_ENTER) {
					showEdit(false);
				}
			}
		});
		jfrEdit.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) { showEdit(false); }
		});
		jfrEdit.setContentPane(jplEdit);
		jfrEdit.pack();
	}
	
	//protected
	//private
	
// methods
	//public
	public void showEdit(boolean blnShow) {
		if (blnShow) setEditValues();
		Point p = new Point(0, getHeight());
		SwingUtilities.convertPointToScreen(p, this);
		if (p.x + jfrEdit.getWidth() > SCREEN_SIZE.width) p.x -= (p.x + jfrEdit.getWidth() - SCREEN_SIZE.width);
		if (p.y + jfrEdit.getHeight() > SCREEN_SIZE.height) p.y -= (p.y + jfrEdit.getHeight() - SCREEN_SIZE.height);
		jfrEdit.setLocation(p);
		jfrEdit.setVisible(blnShow);
	}		
	
	public boolean isEdit() { return jfrEdit.isVisible(); }
	
	//protected
	protected void setViewValues(int[][] intValue) {
		for (int i = 0; i < 2; i++) {
			for (int j = 0; j < 2; j++) {
				intVierFelder[i][j] = intValue[i][j];				
				jlbVierFelder[i][j].setText(Integer.toString(intVierFelder[i][j]));
			}
		}
		jlbViewVierFelderSummary[0].setText(Integer.toString(intVierFelder[0][0] + intVierFelder[0][1]));
		jlbViewVierFelderSummary[1].setText(Integer.toString(intVierFelder[1][0] + intVierFelder[1][1]));
		jlbViewVierFelderSummary[2].setText(Integer.toString(intVierFelder[0][0] + intVierFelder[1][0]));
		jlbViewVierFelderSummary[3].setText(Integer.toString(intVierFelder[0][1] + intVierFelder[1][1]));
		jlbViewVierFelderSummary[4].setText(Integer.toString
			(intVierFelder[0][0] + intVierFelder[0][1] + intVierFelder[1][0] + intVierFelder[1][1])
		);
	}		
	
	protected void getEditValues() {
		if (isInputOK()) mdlVierFelder.setValues(intVierFelderEdit);
	}
		
	protected void setEditValues() {
		for (int i = 0; i < 2; i++) {
			for (int j = 0; j < 2; j++) {
				intVierFelderEdit[i][j] = intVierFelder[i][j];
				jtfVierFelder[i][j].setText(Integer.toString(intVierFelder[i][j]));
			}
		}
		calcSummary();
	}
	
	protected void checkInput(int i, int j) {
		try {
			int x = Integer.parseInt(jtfVierFelder[i][j].getText());
			if (x < 0) throw new NumberFormatException("value must not be negative");
			intVierFelderEdit[i][j] = x;
		} catch(NumberFormatException e) {
			intVierFelderEdit[i][j] = -1;
		}
		calcSummary();
	}			
	
	protected boolean isInputOK() {
		return (
			(intVierFelderEdit[0][0] != -1) &&
			(intVierFelderEdit[0][1] != -1) &&
			(intVierFelderEdit[1][0] != -1) &&
			(intVierFelderEdit[1][1] != -1)
		);
	}	
	
	protected void calcSummary() {
		int f1, f2;
		f1 = intVierFelderEdit[0][0];
		f2 = intVierFelderEdit[0][1];
		jlbVierFelderSummary[0].setText(((f1 == -1) || (f2 == -1))?"n/a":Integer.toString(f1 + f2));
		f1 = intVierFelderEdit[1][0];
		f2 = intVierFelderEdit[1][1];
		jlbVierFelderSummary[1].setText(((f1 == -1) || (f2 == -1))?"n/a":Integer.toString(f1 + f2));
		f1 = intVierFelderEdit[0][0];
		f2 = intVierFelderEdit[1][0];
		jlbVierFelderSummary[2].setText(((f1 == -1) || (f2 == -1))?"n/a":Integer.toString(f1 + f2));
		f1 = intVierFelderEdit[0][1];
		f2 = intVierFelderEdit[1][1];
		jlbVierFelderSummary[3].setText(((f1 == -1) || (f2 == -1))?"n/a":Integer.toString(f1 + f2));
		jlbVierFelderSummary[4].setText((!isInputOK())?"n/a":Integer.toString(
			intVierFelderEdit[0][0] + intVierFelderEdit[0][1] + intVierFelderEdit[1][0] + intVierFelderEdit[1][1]));
	}
		
//setter/getter-methods
	public int[][] getValue() { return intVierFelder; }
	public void setValue(int[][] intValue) throws NumberFormatException, IndexOutOfBoundsException {
		for (int i = 0; i < 2; i++) {
			for (int j = 0; j < 2; j++) {
				if (intValue[i][j] < 0) throw new NumberFormatException("value must not be negative");
			}
		}
		mdlVierFelder.setValues(intValue);
	}
	
	public int getValue(int i, int j) { 
		if (((i == 0) || (i == 1)) && ((j == 0) || (j == 1))) { return intVierFelder[i][j]; }
		else { return -1; }
	}
	
	public Model getModel() { return mdlVierFelder; }
	
//listener
	public void actionPerformed(ActionEvent e) {
		showEdit(true);
	}
	
	public void propertyChange(PropertyChangeEvent e) {
		int[][] intValue = (int[][])e.getNewValue();		
		setViewValues(intValue);
		if (isEdit()) setEditValues();
	}
}
